"""
Command for interactive chat with an agent.

This command provides an interactive chat interface to converse with an agent.
It maintains a conversation history and supports chat commands.
"""
import json
import typer
from typing import Dict, List, Optional, Any
import requests
from rich.console import Console
import uuid
import os

from automagik.config import settings

# Create app for the chat command
chat_app = typer.Typer(no_args_is_help=True)

# Create a rich console for output
console = Console()

@chat_app.callback()
def chat_callback(
    debug: bool = typer.Option(False, "--debug", help="Enable debug mode", is_flag=True, hidden=True)
):
    """
    Start an interactive chat session with an agent.
    
    Use 'start' command with --agent option to begin chatting:
      automagik-agents agent chat start --agent <agent_name>
    
    Or list available agents first:
      automagik-agents agent chat list
    """
    # If debug flag is set, ensure AUTOMAGIK_LOG_LEVEL is set to DEBUG
    if debug:
        os.environ["AUTOMAGIK_LOG_LEVEL"] = "DEBUG"

def get_api_endpoint(path: str) -> str:
    """Build a consistent API endpoint URL with the correct prefix."""
    # Ensure the path doesn't start with a slash
    if path.startswith("/"):
        path = path[1:]
    
    # Always use /api/v1/ prefix
    if not path.startswith("api/v1/"):
        path = f"api/v1/{path}"
    
    # Build the full URL with server from settings
    # The host and port values are stored in AUTOMAGIK_API_HOST and AUTOMAGIK_API_PORT
    server = f"http://{settings.AUTOMAGIK_API_HOST}:{settings.AUTOMAGIK_API_PORT}"
    if not server.endswith('/'):
        server = f"{server}/"
    url = f"{server}{path}"
    
    return url

def get_available_agents() -> List[Dict[str, Any]]:
    """Get a list of available agents using the API."""
    try:
        # Define the API endpoint for listing agents
        endpoint = get_api_endpoint("agent/list")
        if settings.AUTOMAGIK_LOG_LEVEL == "DEBUG":
            console.print(f"Getting agents from: {endpoint}")
        
        # Prepare headers with API key if available
        headers = {}
        if settings.AUTOMAGIK_API_KEY:
            headers["x-api-key"] = settings.AUTOMAGIK_API_KEY
        
        # Make the API request
        try:
            response = requests.get(endpoint, headers=headers, timeout=10)
            
            # Check if the request was successful
            if response.status_code == 200:
                agents = response.json()
                if settings.AUTOMAGIK_LOG_LEVEL == "DEBUG":
                    console.print(f"Successfully retrieved {len(agents)} agents")
                
                # Convert the API response to a format compatible with the rest of the code
                for agent in agents:
                    # Ensure id field is present (use name as fallback)
                    if "id" not in agent:
                        agent["id"] = agent["name"]
                    
                    # If description is missing, provide a default
                    if "description" not in agent or not agent["description"]:
                        agent["description"] = f"Agent of type {agent.get('type', 'unknown')}"
                    
                    # If model is missing, provide a default
                    if "model" not in agent or not agent["model"]:
                        agent["model"] = "unknown"
                
                return agents
            else:
                console.print(f"Error getting agents: HTTP {response.status_code}", style="bold red")
                if settings.AUTOMAGIK_LOG_LEVEL == "DEBUG":
                    console.print(f"Response: {response.text}", style="red")
                return []
        except requests.exceptions.ConnectionError:
            console.print(f"Connection error: Could not connect to API server at {endpoint}", style="bold red")
            return []
    except Exception as e:
        console.print(f"Error getting agents from API: {str(e)}", style="bold red")
        return []

def list_available_agents() -> None:
    """Print a list of available agents."""
    agents = get_available_agents()
    
    if not agents:
        console.print("No agents available or could not connect to the API.", style="bold red")
        console.print("\n[yellow]Possible reasons:[/]")
        console.print("1. The server might not be running. Start it with:")
        console.print("   [cyan]automagik-agents api start[/]")
        console.print("2. Your API server could be running on a different host/port.")
        console.print(f"   Current server setting: [cyan]{settings.AUTOMAGIK_API_HOST}:{settings.AUTOMAGIK_API_PORT}[/]")
        console.print("3. You might not have added any agents yet.")
        
        console.print("\n[green]Try creating an agent first:[/]")
        console.print("  automagik-agents agent create agent --name my_agent --template simple")
        
        console.print("\n[green]Or check if you can access the API directly:[/]")
        console.print(f"  curl http://{settings.AUTOMAGIK_API_HOST}:{settings.AUTOMAGIK_API_PORT}/api/v1/agent/list -H 'x-api-key: {settings.AUTOMAGIK_API_KEY}'")
        return
    
    console.print("\nAvailable Agents:", style="bold green")
    for i, agent in enumerate(agents, 1):
        name = agent.get("name", "Unknown")
        description = agent.get("description", "No description")
        model = agent.get("model", "Unknown model")
        
        console.print(f"{i}. [bold cyan]{name}[/] - {description} [dim](Model: {model})[/]")
    
    console.print("\nUse the agent name to start a chat session:", style="green")
    console.print("  automagik-agents agent chat start --agent <agent_name>", style="bright_black")

async def get_user_by_id(user_id: Optional[str] = None) -> Dict[str, Any]:
    """Get user data from the API by ID."""
    try:
        # Define the API endpoint
        endpoint = get_api_endpoint(f"users/{user_id}")
        if settings.AUTOMAGIK_LOG_LEVEL == "DEBUG":
            console.print(f"Getting user data from: {endpoint}")
        
        # Prepare headers with API key if available
        headers = {}
        if settings.AUTOMAGIK_API_KEY:
            headers["x-api-key"] = settings.AUTOMAGIK_API_KEY
        
        # Make the API request
        response = requests.get(endpoint, headers=headers, timeout=10)
        
        # Check if the request was successful
        if response.status_code == 200:
            user_data = response.json()
            if settings.AUTOMAGIK_LOG_LEVEL == "DEBUG":
                console.print(f"Successfully retrieved user {user_id} from API")
            return user_data
        else:
            if settings.AUTOMAGIK_LOG_LEVEL == "DEBUG":
                console.print(f"Error getting user by ID {user_id}: HTTP {response.status_code}", style="red")
                console.print("Using fallback user data", style="yellow")
            # Return fallback data with UUID-like user_id if needed
            return {"id": user_id, "email": "user@example.com", "name": "User"}
    except Exception as e:
        if settings.AUTOMAGIK_LOG_LEVEL == "DEBUG":
            console.print(f"Error getting user from API: {str(e)}", style="red")
            console.print("Using fallback user data", style="yellow")
        # Return fallback data with UUID-like user_id if needed
        return {"id": user_id, "email": "user@example.com", "name": "User"}

async def run_agent(agent_name: str, input_message: str, session_name: str = None, user_id: Optional[str] = None) -> dict:
    """Run the agent with the given message using the API."""
    try:
        # Check if debug mode is enabled either via settings or directly from environment variable
        debug_mode = (settings.AUTOMAGIK_LOG_LEVEL == "DEBUG") or (os.environ.get("AUTOMAGIK_LOG_LEVEL") == "DEBUG")
        
        # Define the API endpoint with the correct prefix
        endpoint = get_api_endpoint(f"agent/{agent_name}/run")
        
        # Only show endpoint in debug mode
        if debug_mode:
            console.print(f"Using endpoint: {endpoint}")
        
        # Prepare the payload according to the API's expected format
        payload = {
            "message_content": input_message,
            "context": {"debug": debug_mode},
            "session_origin": "cli"  # Always include session_origin for consistency
        }
        
        # Add user_id to payload only if provided
        if user_id is not None:
            payload["user_id"] = user_id
        
        # Add session_name if provided
        if session_name:
            payload["session_name"] = session_name
            # Include debugging info
            if debug_mode:
                console.print(f"Using session name: {session_name}")
        
        if debug_mode:
            console.print(f"Request payload: {json.dumps(payload, indent=2)}")
        
        # Prepare headers with API key
        headers = {
            "Content-Type": "application/json"
        }
        
        # Add API key to headers if available
        if settings.AUTOMAGIK_API_KEY:
            headers["x-api-key"] = settings.AUTOMAGIK_API_KEY
            
            if debug_mode:
                masked_key = f"{settings.AUTOMAGIK_API_KEY[:4]}...{settings.AUTOMAGIK_API_KEY[-4:]}" if len(settings.AUTOMAGIK_API_KEY) > 8 else "****"
                console.print(f"Using API key: {masked_key}")
        
        # Make the API request
        response = requests.post(endpoint, json=payload, headers=headers, timeout=30)
        
        # Check response status
        if response.status_code == 200:
            result = response.json()
            if debug_mode:
                console.print(f"API Response: {json.dumps(result, indent=2)}")
                if "session_id" in result:
                    console.print(f"Session ID from response: {result['session_id']}")
            return result
        else:
            error_msg = f"API Error: Status {response.status_code}"
            try:
                error_data = response.json()
                if "detail" in error_data:
                    error_msg = f"API Error: {error_data['detail']}"
                    
                    # Detect specific errors related to session name uniqueness
                    if "duplicate key value violates unique constraint" in error_data.get("detail", "") and "sessions_name_key" in error_data.get("detail", ""):
                        error_msg = f"Session name '{session_name}' is already in use but with a different agent. Please use a different session name."
                    
                    # Detect session agent mismatch errors
                    elif "already associated with a different agent" in error_data.get("detail", ""):
                        if debug_mode:
                            console.print("Session agent mismatch error. Will retry with agent ID from the existing session.", style="yellow")
                        # For CLI usage, we want to recover and use the session anyway
                        # Retry without specifying an agent_id to let the server use the existing one
                        retry_payload = payload.copy()
                        # Remove any agent_id if present in context
                        if "agent_id" in retry_payload:
                            del retry_payload["agent_id"]
                        if debug_mode:
                            console.print(f"Retrying with payload: {json.dumps(retry_payload, indent=2)}")
                        
                        # Make the retry request
                        retry_response = requests.post(endpoint, json=retry_payload, headers=headers, timeout=30)
                        if retry_response.status_code == 200:
                            retry_result = retry_response.json()
                            if debug_mode:
                                console.print("Retry successful!", style="green")
                            return retry_result
                        else:
                            error_msg = f"API Error on retry: {retry_response.status_code}"
                            if debug_mode:
                                console.print(f"Retry failed: {retry_response.text}", style="red")
            except Exception:
                error_msg = f"API Error: {response.text}"
            
            console.print(f"{error_msg}", style="bold red")
            return {"error": error_msg}
                
    except Exception as e:
        error_msg = f"Error running agent: {str(e)}"
        console.print(f"{error_msg}", style="bold red")
        return {"error": error_msg}

def display_message(message: str, role: str, tool_calls: List = None, tool_outputs: List = None) -> None:
    """Display a message with proper formatting and panels similar to run_chat.py."""
    # Get terminal width to adjust message formatting
    term_width = console.width
    message_width = min(term_width - 20, 80)  # Keep messages reasonably sized
    
    if role == "user":
        # Skip displaying user messages - they're already shown in the chat loop
        pass
    
    elif role == "assistant":
        # Format tool usage in a compact, readable way
        if tool_calls:
            tool_panel_content = []
            
            for i, tool_call in enumerate(tool_calls):
                tool_name = tool_call.get('tool_name', 'Unknown Tool')
                tool_args = tool_call.get('args', {})
                
                # Format tool arguments nicely
                args_str = ""
                if tool_args:
                    if isinstance(tool_args, dict) and len(tool_args) > 0:
                        args_str = ", ".join(f"{k}={v}" for k, v in tool_args.items())
                    else:
                        args_str = str(tool_args)
                
                # Simplified tool call display
                if args_str:
                    tool_call_str = f"🔍 {tool_name}({args_str})"
                else:
                    tool_call_str = f"🔍 {tool_name}()"
                
                # Find and display matching output if available
                if tool_outputs:
                    matching_output = next(
                        (output for output in tool_outputs if output.get("tool_call_id") == tool_call.get("tool_call_id")),
                        None
                    )
                    if matching_output:
                        output_content = matching_output.get('content', '')
                        # Combine tool call and result in a single entry
                        tool_call_str = f"{tool_call_str} → {output_content}"
                
                tool_panel_content.append(tool_call_str)
            
            # Make tool panel very compact and subtle
            if tool_panel_content:
                from rich.panel import Panel
                from rich import box
                console.print(Panel(
                    "\n".join(tool_panel_content),
                    border_style="dim blue",
                    padding=(0, 1),
                    expand=False,
                    width=message_width
                ), justify="right")
        
        # Render the message in a panel
        from rich.panel import Panel
        from rich import box
        console.print(Panel(
            message,
            box=box.ROUNDED,
            border_style="blue",
            padding=(0, 1),
            expand=False,
            width=message_width
        ), justify="right")
    
    elif role == "system":
        # Make system messages subtle and compact
        from rich.panel import Panel
        console.print(Panel(
            message,
            border_style="dim red",
            padding=(0, 1),
            expand=False
        ))
    else:
        # Fallback for any other role
        console.print(f"[{role}] {message}")

def print_help() -> None:
    """Print help information for chat commands."""
    console.print("\n[bold]Available commands:[/]")
    console.print("[cyan]/help[/] - Show this help message")
    console.print("[cyan]/exit[/] or [cyan]/quit[/] - Exit the chat")
    console.print("[cyan]/new[/] - Start a new session (clears history)")
    console.print("[cyan]/history[/] - Show message history for the current session")
    console.print("[cyan]/clear[/] - Clear the screen")
    console.print("[cyan]/debug[/] - Toggle debug mode")
    console.print("[cyan]/session [name][/] - Set or show the current session name")
    console.print("")

async def chat_loop(agent_name: str, session_name: str = None, user_id: Optional[str] = None) -> None:
    """Run an interactive chat loop with the specified agent."""
    # Check if debug mode is enabled either via settings or directly from environment variable
    debug_mode = (settings.AUTOMAGIK_LOG_LEVEL == "DEBUG") or (os.environ.get("AUTOMAGIK_LOG_LEVEL") == "DEBUG")
    
    current_session_name = session_name
    current_session_id = None
    
    # Get user info if user_id is provided
    if user_id is not None:
        await get_user_by_id(user_id)
    
    # First check if the agent exists
    agents = get_available_agents()
    agent = next((a for a in agents if a["name"].lower() == agent_name.lower()), None)
    
    if not agent:
        console.print(f"Error: Agent '{agent_name}' not found", style="bold red")
        console.print("Available agents:", style="yellow")
        list_available_agents()
        return
    
    # If session_name wasn't provided, generate a random one
    if not current_session_name:
        current_session_name = f"cli-{uuid.uuid4().hex[:8]}"
    
    # Display welcome message in a box
    from rich.panel import Panel
    from rich import box
    
    console.print(Panel(
        f"Welcome to the {agent_name} Agent Chat",
        box=box.HEAVY,
        border_style="green",
        expand=False
    ), justify="center")
    
    console.print(f"Starting chat with [bold cyan]{agent_name}[/]")
    console.print(f"Session: [dim]{current_session_name}[/]")
    console.print("[dim]Type your messages and press Enter to send. Type /help for available commands.[/]")
    console.print("")
    
    # Add an initial greeting from the agent
    try:
        # Process a greeting message
        response = await run_agent(agent_name, "Hello", current_session_name, user_id)
        
        # Get the message content
        message_content = ""
        if "message" in response:
            message_content = response.get("message", "")
        elif "history" in response and "messages" in response["history"]:
            # Find the last assistant message
            messages = response["history"]["messages"]
            assistant_msgs = [msg for msg in messages if msg.get("role") == "assistant"]
            if assistant_msgs:
                message_content = assistant_msgs[-1].get("content", "")
        
        # Only display if we got a message
        if message_content:
            # Display assistant greeting
            display_message(message_content, "assistant")
            console.print("")
    except Exception as e:
        if debug_mode:
            console.print(f"[dim]Error displaying initial greeting: {str(e)}[/dim]")
    
    # Chat loop
    while True:
        try:
            # Get user input with a prompt style matching run_chat.py
            console.print("> ", end="")
            user_message = input().strip()
            
            # Check for commands
            if user_message.startswith("/"):
                command = user_message.lower()
                
                # Exit commands
                if command in ["/exit", "/quit"]:
                    console.print("[italic]Exiting chat session.[/]")
                    break
                
                # Help command
                elif command == "/help":
                    print_help()
                    continue
                
                # New session command
                elif command == "/new":
                    # Generate a new session name
                    current_session_name = f"cli-{uuid.uuid4().hex[:8]}"
                    current_session_id = None
                    console.print(f"[italic]Starting new session: {current_session_name}[/]")
                    continue
                
                # History command - to be implemented
                elif command == "/history":
                    console.print("[italic yellow]History view not implemented yet.[/]")
                    continue
                
                # Clear screen command
                elif command == "/clear":
                    os.system('cls' if os.name == 'nt' else 'clear')
                    continue
                
                # Debug toggle command
                elif command == "/debug":
                    debug_mode = not debug_mode
                    console.print(f"[italic]Debug mode: {'enabled' if debug_mode else 'disabled'}[/]")
                    continue
                
                # Session command
                elif command.startswith("/session"):
                    parts = command.split(maxsplit=1)
                    if len(parts) > 1 and parts[1].strip():
                        # Set new session name
                        current_session_name = parts[1].strip()
                        current_session_id = None
                        console.print(f"[italic]Using session: {current_session_name}[/]")
                    else:
                        # Show current session name
                        console.print(f"[italic]Current session: {current_session_name}[/]")
                    continue
                
                # Unknown command
                else:
                    console.print(f"[italic red]Unknown command: {command}[/]")
                    print_help()
                    continue
            
            # Don't duplicate displaying user message - our input prompt already shows it
            # Process message through the agent
            response = await run_agent(agent_name, user_message, current_session_name, user_id)
            
            # Check for errors
            if "error" in response and response["error"]:
                console.print(f"Error: {response['error']}", style="bold red")
                
                # Handle session-specific errors
                if current_session_name and "already in use" in response["error"]:
                    console.print("\n[yellow]This session name is already being used with a different agent.[/]")
                    console.print("[yellow]You can use /session <new_name> to set a different session name.[/]")
                
                # Continue with the next message
                continue
            
            # Store session ID if provided
            if "session_id" in response and response["session_id"]:
                current_session_id = response["session_id"]
                if debug_mode:
                    console.print(f"[dim]Session ID: {current_session_id}[/]")
            
            # Extract response parts
            message_content = ""
            tool_calls = []
            tool_outputs = []
            
            # Check for different response formats and adapt accordingly
            if "message" in response:
                # Direct message in response
                message_content = response.get("message", "")
                
                # Look for tool information in history
                if "history" in response and "messages" in response["history"]:
                    # Find the last assistant message in history
                    messages = response["history"]["messages"]
                    
                    for msg in reversed(messages):
                        if msg.get("role") == "assistant":
                            # If we find a more complete assistant message with tools, use that
                            tool_calls = msg.get("tool_calls", [])
                            tool_outputs = msg.get("tool_outputs", [])
                            break
            elif "history" in response and "messages" in response["history"]:
                # If no direct message, look in history
                messages = response["history"]["messages"]
                
                # Find only the assistant message we care about - skip user messages entirely
                assistant_msgs = [msg for msg in messages if msg.get("role") == "assistant"]
                if assistant_msgs:
                    # Get the last assistant message
                    last_assistant_msg = assistant_msgs[-1]
                    message_content = last_assistant_msg.get("content", "")
                    tool_calls = last_assistant_msg.get("tool_calls", [])
                    tool_outputs = last_assistant_msg.get("tool_outputs", [])
            
            # Display assistant response
            display_message(message_content, "assistant", tool_calls, tool_outputs)
            console.print("")  # Add a blank line after each response for better readability
            
        except KeyboardInterrupt:
            console.print("\n[italic]Chat session interrupted. Exiting...[/]")
            break
        except EOFError:
            console.print("\n[italic]End of input. Exiting...[/]")
            break
        except Exception as e:
            console.print(f"\n[bold red]Error:[/] {str(e)}")
            console.print("[italic]Try again or type /exit to quit.[/]")

@chat_app.command()
def start(
    agent: str = typer.Option(..., "--agent", "-a", help="Agent to chat with"),
    session: Optional[str] = typer.Option(None, "--session", "-s", help="Session name to use/create"),
    user: Optional[str] = typer.Option(None, "--user", "-u", help="User ID (UUID) to use"),
    debug: bool = typer.Option(False, "--debug", help="Enable debug mode", is_flag=True, hidden=True)
):
    """
    Start an interactive chat session with an agent.
    
    This opens a conversational interface where you can talk to the agent
    and receive responses. The conversation history is preserved within
    the session.
    """
    # If debug flag is set, ensure AUTOMAGIK_LOG_LEVEL is set to DEBUG
    if debug:
        os.environ["AUTOMAGIK_LOG_LEVEL"] = "DEBUG"
        
    try:
        import asyncio
        asyncio.run(chat_loop(
            agent_name=agent,
            session_name=session,
            user_id=user
        ))
    except Exception as e:
        console.print(f"[bold red]Error:[/] {str(e)}")
        raise typer.Exit(code=1)

@chat_app.command()
def list():
    """
    List all available agents that can be used for chat.
    """
    list_available_agents()

def get_chats(agent_name: str = None) -> list:
    """Get all chats from the API."""
    try:
        # Check if debug mode is enabled either via settings or directly from environment variable
        debug_mode = (settings.AUTOMAGIK_LOG_LEVEL == "DEBUG") or (os.environ.get("AUTOMAGIK_LOG_LEVEL") == "DEBUG")
        
        # Define the API endpoint with the correct prefix
        endpoint = get_api_endpoint("chats")
        if agent_name:
            endpoint = f"{endpoint}?agent_name={agent_name}"
        
        # Only show endpoint in debug mode
        if debug_mode:
            console.print(f"Using endpoint: {endpoint}")
        
        # Prepare headers with API key
        headers = {}
        if settings.AUTOMAGIK_API_KEY:
            headers["x-api-key"] = settings.AUTOMAGIK_API_KEY
            
            if debug_mode:
                masked_key = f"{settings.AUTOMAGIK_API_KEY[:4]}...{settings.AUTOMAGIK_API_KEY[-4:]}" if len(settings.AUTOMAGIK_API_KEY) > 8 else "****"
                console.print(f"Using API key: {masked_key}")
        
        # Make the API request
        response = requests.get(endpoint, headers=headers, timeout=30)
        
        # Check response status
        if response.status_code == 200:
            result = response.json()
            
            if debug_mode:
                console.print(f"Successfully retrieved {len(result)} chats")
            
            return result
        else:
            error_msg = f"API Error: Status {response.status_code}"
            try:
                error_data = response.json()
                if "detail" in error_data:
                    error_msg = f"API Error: {error_data['detail']}"
            except Exception:
                error_msg = f"API Error: {response.text}"
            
            console.print(f"{error_msg}", style="bold red")
            if debug_mode:
                console.print(f"Response content: {response.text}")
            
            return []
    except Exception as e:
        console.print(f"Error getting chats: {str(e)}", style="bold red")
        if debug_mode:
            import traceback
            console.print(traceback.format_exc())
        return []

def get_chat(session_id: str) -> dict:
    """Get a specific chat by ID from the API."""
    try:
        # Check if debug mode is enabled either via settings or directly from environment variable
        debug_mode = (settings.AUTOMAGIK_LOG_LEVEL == "DEBUG") or (os.environ.get("AUTOMAGIK_LOG_LEVEL") == "DEBUG")
        
        # Define the API endpoint with the correct prefix
        endpoint = get_api_endpoint(f"chats/{session_id}")
        
        # Only show endpoint in debug mode
        if debug_mode:
            console.print(f"Using endpoint: {endpoint}")
        
        # Prepare headers with API key
        headers = {}
        if settings.AUTOMAGIK_API_KEY:
            headers["x-api-key"] = settings.AUTOMAGIK_API_KEY
            
            if debug_mode:
                masked_key = f"{settings.AUTOMAGIK_API_KEY[:4]}...{settings.AUTOMAGIK_API_KEY[-4:]}" if len(settings.AUTOMAGIK_API_KEY) > 8 else "****"
                console.print(f"Using API key: {masked_key}")
        
        # Make the API request
        response = requests.get(endpoint, headers=headers, timeout=30)
        
        # Check response status
        if response.status_code == 200:
            result = response.json()
            
            if debug_mode:
                console.print(f"Successfully retrieved chat with ID: {session_id}")
            
            return result
        else:
            error_msg = f"API Error: Status {response.status_code}"
            try:
                error_data = response.json()
                if "detail" in error_data:
                    error_msg = f"API Error: {error_data['detail']}"
            except Exception:
                error_msg = f"API Error: {response.text}"
            
            console.print(f"{error_msg}", style="bold red")
            if debug_mode:
                console.print(f"Response content: {response.text}")
            
            return {}
    except Exception as e:
        console.print(f"Error getting chat: {str(e)}", style="bold red")
        if debug_mode:
            import traceback
            console.print(traceback.format_exc())
        return {}

def delete_chat(session_id: str) -> bool:
    """Delete a specific chat by ID using the API."""
    try:
        # Check if debug mode is enabled either via settings or directly from environment variable
        debug_mode = (settings.AUTOMAGIK_LOG_LEVEL == "DEBUG") or (os.environ.get("AUTOMAGIK_LOG_LEVEL") == "DEBUG")
        
        # Define the API endpoint with the correct prefix
        endpoint = get_api_endpoint(f"chats/{session_id}")
        
        # Only show endpoint in debug mode
        if debug_mode:
            console.print(f"Using endpoint for DELETE: {endpoint}")
        
        # Prepare headers with API key
        headers = {}
        if settings.AUTOMAGIK_API_KEY:
            headers["x-api-key"] = settings.AUTOMAGIK_API_KEY
            
            if debug_mode:
                masked_key = f"{settings.AUTOMAGIK_API_KEY[:4]}...{settings.AUTOMAGIK_API_KEY[-4:]}" if len(settings.AUTOMAGIK_API_KEY) > 8 else "****"
                console.print(f"Using API key: {masked_key}")
        
        # Make the API request
        response = requests.delete(endpoint, headers=headers, timeout=30)
        
        # Check response status
        if response.status_code == 200:
            if debug_mode:
                console.print(f"Successfully deleted chat with ID: {session_id}")
            
            return True
        else:
            error_msg = f"API Error: Status {response.status_code}"
            try:
                error_data = response.json()
                if "detail" in error_data:
                    error_msg = f"API Error: {error_data['detail']}"
            except Exception:
                error_msg = f"API Error: {response.text}"
            
            console.print(f"{error_msg}", style="bold red")
            if debug_mode:
                console.print(f"Response content: {response.text}")
            
            return False
    except Exception as e:
        console.print(f"Error deleting chat: {str(e)}", style="bold red")
        if debug_mode:
            import traceback
            console.print(traceback.format_exc())
        return False 


import logging
import uuid
from fastapi import APIRouter, HTTPException, Query, Path
from automagik.api.models import SessionListResponse, SessionBranchesResponse, SessionBranchTreeResponse
from automagik.api.controllers.session_controller import get_sessions, get_session, delete_session, get_session_branches_controller, get_session_branch_tree_controller

# Create router for session endpoints
session_router = APIRouter()

# Get our module's logger
logger = logging.getLogger(__name__)

@session_router.get("/sessions", response_model=SessionListResponse, tags=["Sessions"],
            summary="List All Sessions",
            description="Retrieve a list of all sessions with pagination options.")
async def list_sessions_route(
    page: int = Query(1, ge=1, description="Page number"),
    page_size: int = Query(50, ge=1, le=100, description="Items per page"),
    sort_desc: bool = Query(True, description="Sort by most recent first")
):
    """
    Get a paginated list of all sessions
    """
    return await get_sessions(page, page_size, sort_desc)

@session_router.get("/sessions/{session_id_or_name}", tags=["Sessions"],
           summary="Get Session History",
           description="Retrieve a session's message history with pagination options. You can use either the session ID (UUID) or a session name.")
async def get_session_route(
    session_id_or_name: str,
    page: int = Query(1, ge=1, description="Page number"),
    page_size: int = Query(50, ge=1, le=100, description="Items per page"),
    sort_desc: bool = Query(True, description="Sort by most recent first"),
    hide_tools: bool = Query(False, description="Exclude tool calls and outputs"),
    show_system_prompt: bool = Query(False, description="Include system prompt in session details")
):
    """
    Get a session by ID or name with its message history
    """
    try:
        session_data = await get_session(session_id_or_name, page, page_size, sort_desc, hide_tools, show_system_prompt)
        
        # For name lookups, return the name as the session_id
        session_name = session_data["session"].session_name
        session_id = session_data["session"].session_id
        response_id = session_id_or_name if session_id_or_name == session_name else session_id
        
        # Prepare the session details from SessionInfo
        # Use .model_dump() for Pydantic v2, or .dict() for v1
        # Assuming Pydantic v2+ for .model_dump()
        session_details = session_data["session"].model_dump(exclude_none=True)
        
        # Ensure all required fields are strings if they are UUID or datetime
        if 'user_id' in session_details and session_details['user_id'] is not None:
            session_details['user_id'] = str(session_details['user_id'])
        if 'created_at' in session_details and session_details['created_at'] is not None:
            session_details['created_at'] = session_details['created_at'].isoformat()
        if 'last_updated' in session_details and session_details['last_updated'] is not None:
            session_details['last_updated'] = session_details['last_updated'].isoformat()

        # Construct the final response
        response_payload = {
            **session_details, # Spread all fields from SessionInfo
            "session_id": response_id, # Override session_id with the one determined for the route
            "messages": session_data["messages"],
            "exists": True,
            "total_messages": session_data["total"], # This is the same as session_details.get('message_count')
            "current_page": session_data["page"],
            "total_pages": session_data["total_pages"],
            "system_prompt": session_data.get("system_prompt")
        }
        return response_payload
    except HTTPException as e:
        if e.status_code == 404:
            # Return 404 status code when session not found, don't handle it
            raise
        # Rethrow other exceptions
        raise

@session_router.delete("/sessions/{session_id_or_name}", tags=["Sessions"],
              summary="Delete Session",
              description="Delete a session's message history by its ID or name.")
async def delete_session_route(session_id_or_name: str):
    """
    Delete a session by ID or name
    """
    await delete_session(session_id_or_name)
    return {
        "status": "success",
        "session_id": session_id_or_name,
        "message": f"Session {session_id_or_name} deleted successfully"
    }


@session_router.get(
    "/sessions/{session_id}/branches",
    response_model=SessionBranchesResponse,
    tags=["Sessions", "Branching"],
    summary="Get Session Branches",
    description="Retrieve all conversation branches for a specific session."
)
async def get_session_branches_route(
    session_id: uuid.UUID = Path(..., description="The unique identifier of the session.")
):
    """
    Get all conversation branches for a session.
    
    Returns the main session and all its branches, including metadata
    about each branch such as the branch point message and creation time.
    """
    try:
        response_data = await get_session_branches_controller(session_id)
        return SessionBranchesResponse(**response_data)
    except HTTPException as e:
        raise e
    except Exception as e:
        logger.error(f"Unexpected error in get_session_branches_route for session_id {session_id}: {str(e)}")
        raise HTTPException(status_code=500, detail="An unexpected error occurred while trying to get session branches.")


@session_router.get(
    "/sessions/{session_id}/branch-tree",
    response_model=SessionBranchTreeResponse,
    tags=["Sessions", "Branching"],
    summary="Get Session Branch Tree",
    description="Retrieve the complete conversation branch tree for a session as a hierarchical structure."
)
async def get_session_branch_tree_route(
    session_id: uuid.UUID = Path(..., description="The unique identifier of the session.")
):
    """
    Get the complete conversation branch tree for a session.
    
    Returns a hierarchical tree structure showing the relationship
    between the main session and all its branches and sub-branches.
    """
    try:
        response_data = await get_session_branch_tree_controller(session_id)
        return SessionBranchTreeResponse(**response_data)
    except HTTPException as e:
        raise e
    except Exception as e:
        logger.error(f"Unexpected error in get_session_branch_tree_route for session_id {session_id}: {str(e)}")
        raise HTTPException(status_code=500, detail="An unexpected error occurred while trying to get session branch tree.") 


"""Enhanced Summary Agent using framework patterns."""
from typing import Dict

from automagik.agents.pydanticai.simple.agent import SimpleAgent as BaseSimpleAgent
from .prompts.prompt import AGENT_PROMPT


class SummaryAgent(BaseSimpleAgent):
    """Enhanced Summary Agent with Claude Sonnet-4 model."""
    
    def __init__(self, config: Dict[str, str]) -> None:
        """Initialize Summary Agent with automatic setup."""
        super().__init__(config)
        
        # Set the prompt text
        self._code_prompt_text = AGENT_PROMPT
        
        # Override to use Claude Sonnet-4
        if hasattr(self.dependencies, 'model_name'):
            self.dependencies.model_name = "anthropic:claude-3-5-sonnet-20241022"


def create_agent(config: Dict[str, str]) -> SummaryAgent:
    """Factory function to create enhanced Summary agent."""
    try:
        return SummaryAgent(config)
    except Exception:
        from automagik.agents.models.placeholder import PlaceholderAgent
        return PlaceholderAgent(config)


"""Gmail tool schemas.

This module defines the Pydantic models for Gmail tool input and output.
"""
from pydantic import BaseModel, Field
from typing import Optional, List, Dict, Any

class SendEmailInput(BaseModel):
    """Input model for sending email."""
    to: str = Field(..., description="Email address of the recipient")
    cc: Optional[List[str]] = Field(None, description="List of email addresses to CC")
    subject: str = Field(..., description="Subject of the email")
    message: str = Field(..., description="Content of the email")
    content_type: Optional[str] = Field("text/plain", description="Content type of the email: text/plain or text/html")
    plain_text_alternative: Optional[str] = Field(None, description="Plain text alternative for HTML emails")
    extra_content: Optional[str] = Field(None, description="Additional content to append to the email message")

class SendEmailResult(BaseModel):
    """Result model for sending email."""
    success: bool = Field(..., description="Whether the email was sent successfully")
    message_id: Optional[str] = Field(None, description="ID of the sent message if successful")
    error: Optional[str] = Field(None, description="Error message if sending failed")

class FetchEmailsInput(BaseModel):
    """Input model for fetching emails."""
    subject_filter: Optional[str] = Field(None, description="Filter emails by subject (e.g. '[STAN] - Novo Lead')")
    max_results: int = Field(10, description="Maximum number of emails to retrieve")

class EmailMessage(BaseModel):
    """Model for an email message."""
    id: str = Field(..., description="ID of the email message")
    from_email: str = Field(..., description="Sender email address")
    to: str = Field(..., description="Recipient email address")
    subject: str = Field(..., description="Subject of the email")
    date: str = Field(..., description="Date the email was sent")
    body: str = Field(..., description="Body content of the email")
    raw_data: Dict[str, Any] = Field(..., description="Raw message data from Gmail API")

class FetchEmailsResult(BaseModel):
    """Result model for fetching emails."""
    success: bool = Field(..., description="Whether the emails were fetched successfully")
    emails: List[EmailMessage] = Field(default_factory=list, description="List of fetched email messages")
    error: Optional[str] = Field(None, description="Error message if fetching failed") 


"""PydanticAI agents package.

This package contains agents built with PydanticAI framework.
(Previously located at src.agents.simple)
"""

import os
import importlib
import logging
from pathlib import Path
from importlib import import_module

# Setup logging
logger = logging.getLogger(__name__)

# Discover agents in subfolders
def discover_agents():
    """Discover agent modules in the pydanticai agent type directory."""
    agents = {}
    current_dir = Path(__file__).parent
    
    for item in current_dir.iterdir():
        if item.is_dir() and not item.name.startswith('__') and not item.name.startswith('.'):
            try:
                # Try to import the module
                module_name = f"automagik.agents.pydanticai.{item.name}"
                module = importlib.import_module(module_name)
                
                # Check if the module has a create_agent function
                if hasattr(module, "create_agent") and callable(module.create_agent):
                    agent_name = item.name
                    agents[agent_name] = module.create_agent
            except Exception as e:
                logger.error(f"Error importing agent from {item.name}: {str(e)}")
    
    return agents

# Get discovered agents
_discovered_agents = discover_agents()

def create_agent(agent_name=None):
    """Create an agent instance by name.
    
    Args:
        agent_name: The name of the agent to create
                   If None, creates a simple agent.
        
    Returns:
        An instance of the requested agent
    
    Raises:
        ValueError: If the agent cannot be found or created
    """
    # If no agent_name specified or it's "simple", default to simple
    if agent_name is None or agent_name == "simple":
        agent_name = "simple"
    
    # Remove _agent suffix if present (for normalization)
    if agent_name.endswith("_agent"):
        base_name = agent_name
    else:
        base_name = f"{agent_name}_agent"
    
    logger.info(f"Creating agent: {base_name}")
    
    # Try to find the agent in discovered agents
    if base_name in _discovered_agents:
        return _discovered_agents[base_name]()
    
    # Direct import approach if agent wasn't discovered
    try:
        module_path = f"automagik.agents.pydanticai.{base_name}"
        module = importlib.import_module(module_path)
        
        if hasattr(module, "create_agent"):
            return module.create_agent()
        else:
            raise ValueError(f"Module {module_path} has no create_agent function")
            
    except ImportError as e:
        raise ValueError(f"Could not import agent module for {base_name}: {str(e)}")
    except Exception as e:
        raise ValueError(f"Error creating agent {base_name}: {str(e)}")

# Canonical exports for common agent classes
SimpleAgent = import_module("automagik.agents.pydanticai.simple.agent").SimpleAgent  # type: ignore
# StanAgent = import_module("automagik.agents.pydanticai.stan.agent").StanAgent  # type: ignore  # Commented out - missing module
# StanEmailAgent = import_module("automagik.agents.pydanticai.stan_email.agent").StanEmailAgent  # type: ignore  # Commented out - missing module
SofiaAgent = import_module("automagik.agents.pydanticai.sofia.agent").SofiaAgent  # type: ignore
SummaryAgent = import_module("automagik.agents.pydanticai.summary.agent").SummaryAgent  # type: ignore
PromptMakerAgent = import_module("automagik.agents.pydanticai.prompt_maker.agent").PromptMakerAgent  # type: ignore

__all__ = [
    "SimpleAgent",
    # "StanAgent",  # Commented out - missing module
    # "StanEmailAgent",  # Commented out - missing module
    "SofiaAgent",
    "SummaryAgent",
    "PromptMakerAgent",
]



"""
SDK Stream Processor for extracting rich real-time data from Claude Code SDK.

This module processes the streaming messages from the claude-code-sdk to extract
detailed execution metrics, progress tracking, and status information that
feeds the status endpoint with rich details.
"""

import logging
import sys
import time
from datetime import datetime
from pathlib import Path
from typing import Dict, List, Any, Optional
from dataclasses import dataclass, field

# Add local SDK to path if available
local_sdk_path = Path(__file__).parent.parent.parent / "vendors" / "claude-code-sdk" / "src"
if local_sdk_path.exists():
    sys.path.insert(0, str(local_sdk_path))

from claude_code_sdk import (
    AssistantMessage, 
    UserMessage, 
    SystemMessage, 
    ResultMessage,
    TextBlock,
    ToolUseBlock,
    ToolResultBlock
)

logger = logging.getLogger(__name__)


@dataclass
class SDKExecutionMetrics:
    """Comprehensive metrics extracted from SDK streaming data."""
    
    # Basic execution info
    session_id: Optional[str] = None
    total_turns: int = 0
    total_cost_usd: float = 0.0
    total_tokens: int = 0
    duration_ms: int = 0
    duration_api_ms: int = 0
    
    # Tool usage tracking
    tool_calls: int = 0
    tool_names_used: List[str] = field(default_factory=list)
    tool_usage_history: List[Dict[str, Any]] = field(default_factory=list)
    
    # Content tracking
    assistant_responses: List[str] = field(default_factory=list)
    user_messages: List[str] = field(default_factory=list)
    system_events: List[Dict[str, Any]] = field(default_factory=list)
    
    # Status tracking
    is_running: bool = False
    is_success: bool = False
    completion_type: str = "running"
    final_result: Optional[str] = None
    error_message: Optional[str] = None
    
    # Real-time progress
    current_phase: str = "initializing"
    phases_completed: List[str] = field(default_factory=list)
    last_activity: Optional[datetime] = None
    
    # Model and MCP info
    model: Optional[str] = None
    mcp_servers: List[str] = field(default_factory=list)


class SDKStreamProcessor:
    """Processes Claude Code SDK streaming messages for real-time metrics."""
    
    def __init__(self):
        """Initialize the stream processor."""
        self.metrics = SDKExecutionMetrics()
        self.start_time = time.time()
        self.current_turn = 0
        self.expecting_tool_result = False
        self.current_tool_use_id = None
        
    def process_message(self, message) -> Dict[str, Any]:
        """
        Process a streaming message from the SDK and update metrics.
        
        Args:
            message: Message object from claude-code-sdk
            
        Returns:
            Dict containing the processed event data
        """
        self.metrics.last_activity = datetime.utcnow()
        
        if isinstance(message, UserMessage):
            return self._process_user_message(message)
        elif isinstance(message, AssistantMessage):
            return self._process_assistant_message(message)
        elif isinstance(message, SystemMessage):
            return self._process_system_message(message)
        elif isinstance(message, ResultMessage):
            return self._process_result_message(message)
        else:
            logger.debug(f"Unknown message type: {type(message)}")
            return {"event_type": "unknown", "data": str(message)}
    
    def _process_user_message(self, message: UserMessage) -> Dict[str, Any]:
        """Process user message."""
        self.metrics.user_messages.append(message.content)
        
        return {
            "event_type": "user_message",
            "timestamp": datetime.utcnow().isoformat(),
            "data": {
                "content": message.content,
                "message_count": len(self.metrics.user_messages)
            }
        }
    
    def _process_assistant_message(self, message: AssistantMessage) -> Dict[str, Any]:
        """Process assistant message with content blocks."""
        events = []
        
        for block in message.content:
            if isinstance(block, TextBlock):
                # Track assistant text response
                self.metrics.assistant_responses.append(block.text)
                
                # Update progress based on content
                self._update_phase_from_content(block.text)
                
                events.append({
                    "event_type": "assistant_text",
                    "timestamp": datetime.utcnow().isoformat(),
                    "data": {
                        "text": block.text,
                        "response_count": len(self.metrics.assistant_responses),
                        "current_phase": self.metrics.current_phase
                    }
                })
                
            elif isinstance(block, ToolUseBlock):
                # Track tool usage
                self.metrics.tool_calls += 1
                if block.name not in self.metrics.tool_names_used:
                    self.metrics.tool_names_used.append(block.name)
                
                tool_event = {
                    "tool_id": block.id,
                    "tool_name": block.name,
                    "input": block.input,
                    "timestamp": datetime.utcnow().isoformat()
                }
                self.metrics.tool_usage_history.append(tool_event)
                
                # Extract phases from TodoWrite tool usage
                if block.name == "TodoWrite":
                    self._extract_phases_from_todowrite(block.input)
                
                # Update phase for tool usage
                self.metrics.current_phase = f"using_{block.name.lower()}_tool"
                
                events.append({
                    "event_type": "tool_use",
                    "timestamp": datetime.utcnow().isoformat(),
                    "data": {
                        "tool_id": block.id,
                        "tool_name": block.name,
                        "input": block.input,
                        "tool_calls_total": self.metrics.tool_calls,
                        "unique_tools": len(self.metrics.tool_names_used)
                    }
                })
                
                self.expecting_tool_result = True
                self.current_tool_use_id = block.id
                
            elif isinstance(block, ToolResultBlock):
                # Track tool result
                events.append({
                    "event_type": "tool_result",
                    "timestamp": datetime.utcnow().isoformat(),
                    "data": {
                        "tool_use_id": block.tool_use_id,
                        "content": block.content,
                        "is_error": block.is_error,
                        "success": not block.is_error
                    }
                })
                
                self.expecting_tool_result = False
                self.current_tool_use_id = None
                
                # Update phase after tool completion
                if not block.is_error:
                    self.metrics.current_phase = "processing_tool_result"
                else:
                    self.metrics.current_phase = "handling_tool_error"
        
        # If multiple events, return the most significant one
        if events:
            return events[-1]  # Return the last (most recent) event
        
        return {
            "event_type": "assistant_message",
            "timestamp": datetime.utcnow().isoformat(),
            "data": {"content_blocks": len(message.content)}
        }
    
    def _process_system_message(self, message: SystemMessage) -> Dict[str, Any]:
        """Process system message."""
        system_event = {
            "subtype": message.subtype,
            "data": message.data,
            "timestamp": datetime.utcnow().isoformat()
        }
        self.metrics.system_events.append(system_event)
        
        # Update metrics based on system message
        if message.subtype == "session_start":
            self.metrics.session_id = message.data.get("session_id")
            self.metrics.is_running = True
            self.metrics.current_phase = "session_started"
        elif message.subtype == "turn_start":
            self.current_turn += 1
            self.metrics.total_turns = self.current_turn
            self.metrics.current_phase = f"turn_{self.current_turn}"
        
        return {
            "event_type": "system_message",
            "timestamp": datetime.utcnow().isoformat(),
            "data": {
                "subtype": message.subtype,
                "system_data": message.data,
                "current_turn": self.current_turn
            }
        }
    
    def _process_result_message(self, message: ResultMessage) -> Dict[str, Any]:
        """Process result message with final metrics."""
        
        # Update final metrics
        self.metrics.total_cost_usd = message.total_cost_usd or 0.0
        self.metrics.duration_ms = message.duration_ms
        self.metrics.duration_api_ms = message.duration_api_ms
        self.metrics.total_turns = message.num_turns
        self.metrics.session_id = message.session_id
        self.metrics.final_result = message.result
        self.metrics.is_success = not message.is_error
        self.metrics.is_running = False
        
        # Extract token usage if available
        if message.usage:
            self.metrics.total_tokens = message.usage.get("total_tokens", 0)
        
        # Determine completion type and phase
        if message.is_error:
            self.metrics.completion_type = "failed"
            self.metrics.current_phase = "failed"
            self.metrics.error_message = message.result
        else:
            if message.subtype == "max_turns_reached":
                self.metrics.completion_type = "max_turns_reached"
            else:
                self.metrics.completion_type = "completed_successfully"
            self.metrics.current_phase = "completed"
        
        return {
            "event_type": "execution_complete",
            "timestamp": datetime.utcnow().isoformat(),
            "data": {
                "success": self.metrics.is_success,
                "completion_type": self.metrics.completion_type,
                "total_cost_usd": self.metrics.total_cost_usd,
                "duration_ms": self.metrics.duration_ms,
                "total_turns": self.metrics.total_turns,
                "session_id": self.metrics.session_id,
                "result": self.metrics.final_result,
                "is_error": message.is_error
            }
        }
    
    def _update_phase_from_content(self, text: str):
        """Update execution phase based on assistant text content."""
        text_lower = text.lower()
        
        # Common phase detection patterns
        if any(word in text_lower for word in ["analyzing", "examining", "looking at"]):
            self.metrics.current_phase = "analysis"
            if "analysis" not in self.metrics.phases_completed:
                self.metrics.phases_completed.append("analysis")
                
        elif any(word in text_lower for word in ["planning", "will", "going to", "approach"]):
            self.metrics.current_phase = "planning"
            if "planning" not in self.metrics.phases_completed:
                self.metrics.phases_completed.append("planning")
                
        elif any(word in text_lower for word in ["implementing", "creating", "writing", "coding"]):
            self.metrics.current_phase = "implementation"
            if "implementation" not in self.metrics.phases_completed:
                self.metrics.phases_completed.append("implementation")
                
        elif any(word in text_lower for word in ["testing", "running test", "checking"]):
            self.metrics.current_phase = "testing"
            if "testing" not in self.metrics.phases_completed:
                self.metrics.phases_completed.append("testing")
                
        elif any(word in text_lower for word in ["completed", "finished", "done"]):
            self.metrics.current_phase = "completion"
            if "completion" not in self.metrics.phases_completed:
                self.metrics.phases_completed.append("completion")
        
        # Track phase progress (percentage calculation removed per user request)
    
    def _extract_phases_from_todowrite(self, tool_input: Dict[str, Any]):
        """Extract phase information from TodoWrite tool usage."""
        if not isinstance(tool_input, dict):
            return
            
        todos = tool_input.get("todos", [])
        if not isinstance(todos, list):
            return
        
        completed_phases = []
        current_phase = None
        
        for todo in todos:
            if not isinstance(todo, dict):
                continue
                
            content = todo.get("content", "")
            status = todo.get("status", "")
            
            if status == "completed" and content:
                # Clean and format phase name
                phase_name = content.strip()
                if phase_name and phase_name not in completed_phases:
                    completed_phases.append(phase_name)
            elif status == "in_progress" and content:
                # Set current active phase
                current_phase = content.strip()
        
        # Update metrics with extracted phase information
        if completed_phases:
            # Merge with existing phases, preserving order and avoiding duplicates
            for phase in completed_phases:
                if phase not in self.metrics.phases_completed:
                    self.metrics.phases_completed.append(phase)
        
        if current_phase:
            self.metrics.current_phase = current_phase
    
    def get_current_metrics(self) -> SDKExecutionMetrics:
        """Get current execution metrics."""
        return self.metrics
    
    def get_status_data(self) -> Dict[str, Any]:
        """Get formatted status data for the status endpoint."""
        elapsed_seconds = time.time() - self.start_time
        
        return {
            "run_id": "sdk_execution",  # This should be set externally
            "status": "completed" if not self.metrics.is_running else ("running" if self.metrics.total_turns > 0 else "pending"),
            "workflow_name": "unknown",  # This should be set externally
            "started_at": datetime.fromtimestamp(self.start_time).isoformat(),
            "completed_at": self.metrics.last_activity.isoformat() if not self.metrics.is_running and self.metrics.last_activity else None,
            "execution_time_seconds": elapsed_seconds if not self.metrics.is_running else None,
            
            "progress": {
                "turns": self.metrics.total_turns,
                "max_turns": None,  # Should be set externally
                "current_phase": self.metrics.current_phase,
                "phases_completed": self.metrics.phases_completed,
                "is_running": self.metrics.is_running
            },
            
            "metrics": {
                "cost_usd": self.metrics.total_cost_usd,
                "tokens": {
                    "total": self.metrics.total_tokens,
                    "input": 0,  # SDK doesn't provide breakdown yet
                    "output": 0,
                    "cache_created": 0,
                    "cache_read": 0,
                    "cache_efficiency": 0.0
                },
                "tools_used": self.metrics.tool_names_used,
                "tool_names_used": self.metrics.tool_names_used,  # Keep for backwards compatibility
                "api_duration_ms": self.metrics.duration_api_ms,
                "performance_score": self._calculate_performance_score()
            },
            
            "result": {
                "success": self.metrics.is_success,
                "completion_type": self.metrics.completion_type,
                "message": self._get_status_message(),
                "final_output": self.metrics.final_result,
                "files_created": [],  # TODO: Extract from tool usage
                "git_commits": []  # TODO: Extract from tool usage
            }
        }
    
    def _calculate_performance_score(self) -> float:
        """Calculate a performance score based on execution metrics."""
        score = 85.0  # Base score
        
        # Adjust based on tool usage efficiency
        if self.metrics.tool_calls > 0 and self.metrics.total_turns > 0:
            tools_per_turn = self.metrics.tool_calls / self.metrics.total_turns
            if tools_per_turn > 3:  # Too many tools per turn
                score -= 10
            elif tools_per_turn < 0.5:  # Too few tools (might be inefficient)
                score -= 5
        
        # Adjust based on completion
        if self.metrics.is_success:
            score += 5
        elif self.metrics.error_message:
            score -= 15
        
        return max(0.0, min(100.0, score))
    
    def _get_status_message(self) -> str:
        """Get a user-friendly status message."""
        if self.metrics.error_message:
            return f"❌ Execution failed: {self.metrics.error_message}"
        elif not self.metrics.is_running and self.metrics.is_success:
            return "✅ Workflow completed successfully"
        elif not self.metrics.is_running and self.metrics.completion_type == "max_turns_reached":
            return "⏰ Reached maximum turns - workflow stopped at turn limit"
        elif self.metrics.is_running:
            return f"🔄 Running - Turn {self.metrics.total_turns}, {len(self.metrics.tool_names_used)} tools used"
        else:
            return "⚠️ Workflow status unclear - check logs for details"


"""
Comprehensive Usage Calculator for Multimodal AI Operations

This module provides unified cost tracking across frameworks and content types,
addressing the PydanticAI bias and multimodal blindness in current tracking.
"""

import logging
from typing import Dict, Any, List, Optional, Union
from dataclasses import dataclass, asdict
from datetime import datetime
import json

logger = logging.getLogger(__name__)

@dataclass
class MediaUsage:
    """Usage tracking for different media types."""
    text_tokens: int = 0
    image_tokens: int = 0  # Images typically equivalent to ~765 tokens each
    audio_seconds: float = 0.0  # Audio processing time
    video_seconds: float = 0.0  # Video processing time
    preprocessing_ms: float = 0.0  # Multimodal preprocessing overhead

@dataclass 
class UsageBreakdown:
    """Comprehensive usage breakdown for a single request."""
    framework: str
    model: str
    request_timestamp: str
    processing_time_ms: float
    
    # Token-based costs
    input_tokens: int = 0
    output_tokens: int = 0
    cached_tokens: int = 0
    total_tokens: int = 0
    
    # Multimodal usage
    media_usage: MediaUsage = None
    
    # Framework-specific data
    framework_events: List[Dict[str, Any]] = None
    
    # Content type attribution
    content_types: List[str] = None  # ['text', 'image', 'audio', 'video']
    
    def __post_init__(self):
        if self.media_usage is None:
            self.media_usage = MediaUsage()
        if self.framework_events is None:
            self.framework_events = []
        if self.content_types is None:
            self.content_types = []

class MediaTokenConfig:
    """Configuration for multimodal content token equivalents."""
    
    # Token equivalents for different media types
    MEDIA_TOKEN_EQUIVALENTS = {
        "image_token_equivalent": 765,  # GPT-4V: ~765 tokens per image
        "audio_tokens_per_second": 50,  # Estimated tokens per second of audio
        "video_frames_per_second": 1,   # Key frames extracted per second
        "video_tokens_per_frame": 765,  # Each frame = image token equivalent
    }

class UnifiedUsageCalculator:
    """Unified usage calculator that works across all frameworks and content types."""
    
    def __init__(self):
        self.media_config = MediaTokenConfig()
    
    def extract_pydantic_ai_usage(self, result: Any, model: str, processing_time_ms: float, multimodal_content: Optional[Dict[str, Any]] = None) -> UsageBreakdown:
        """Extract usage from PydanticAI result."""
        breakdown = UsageBreakdown(
            framework="pydantic_ai",
            model=model,
            request_timestamp=datetime.utcnow().isoformat(),
            processing_time_ms=processing_time_ms
        )
        
        try:
            # Extract from PydanticAI's detailed usage tracking
            if hasattr(result, 'all_messages'):
                total_input = 0
                total_output = 0
                total_cached = 0
                
                for message in result.all_messages():
                    if hasattr(message, 'usage') and message.usage:
                        total_input += getattr(message.usage, 'request_tokens', 0) or 0
                        total_output += getattr(message.usage, 'response_tokens', 0) or 0
                        # PydanticAI specific: extract cache tokens from details
                        if hasattr(message.usage, 'details') and message.usage.details:
                            cache_creation = getattr(message.usage.details, 'cache_creation_input_tokens', 0) or 0
                            cache_read = getattr(message.usage.details, 'cache_read_input_tokens', 0) or 0
                            total_cached += cache_creation + cache_read
                
                breakdown.input_tokens = total_input
                breakdown.output_tokens = total_output
                breakdown.cached_tokens = total_cached
                breakdown.total_tokens = total_input + total_output
        
        except Exception as e:
            logger.warning(f"Error extracting PydanticAI usage: {e}")
        
        # Set content types dynamically based on multimodal content
        content_types = ["text"]
        if multimodal_content:
            if multimodal_content.get('images'):
                content_types.append("image")
            if multimodal_content.get('audio'):
                content_types.append("audio")
            if multimodal_content.get('videos'):
                content_types.append("video")
            if multimodal_content.get('documents'):
                content_types.append("document")
        
        breakdown.content_types = content_types
        
        return breakdown
    
    def extract_agno_usage(self, result: Any, model: str, processing_time_ms: float, 
                          multimodal_content: Optional[Dict[str, Any]] = None) -> UsageBreakdown:
        """Extract comprehensive usage from Agno result with multimodal awareness."""
        breakdown = UsageBreakdown(
            framework="agno",
            model=model,
            request_timestamp=datetime.utcnow().isoformat(),
            processing_time_ms=processing_time_ms
        )
        
        # Initialize content types
        content_types = ["text"]
        
        try:
            # Extract basic token usage from Agno
            if hasattr(result, 'usage'):
                breakdown.input_tokens = getattr(result.usage, 'prompt_tokens', 0)
                breakdown.output_tokens = getattr(result.usage, 'completion_tokens', 0)
                breakdown.total_tokens = getattr(result.usage, 'total_tokens', 0)
            
            # Extract Agno-specific events for detailed tracking
            if hasattr(result, 'events') and result.events:
                breakdown.framework_events = [
                    {
                        "event": event.event,
                        "timestamp": getattr(event, 'timestamp', None),
                        "tool": getattr(event, 'tool', None),
                        "duration_ms": getattr(event, 'duration_ms', None)
                    }
                    for event in result.events
                ]
            
            # 🎯 MULTIMODAL USAGE TRACKING - Track media usage without pricing
            if multimodal_content:
                media_usage = MediaUsage()
                
                # Track image usage
                images = multimodal_content.get('images', [])
                if images:
                    content_types.append("image")
                    # Each image ~= 765 tokens equivalent
                    media_usage.image_tokens = len(images) * self.media_config.MEDIA_TOKEN_EQUIVALENTS["image_token_equivalent"]
                
                # Track audio usage
                audio_files = multimodal_content.get('audio', [])
                if audio_files:
                    content_types.append("audio")
                    # Estimate audio duration (would need actual duration in real implementation)
                    estimated_duration = len(audio_files) * 10.0  # Assume 10s per file
                    media_usage.audio_seconds = estimated_duration
                
                # Track video usage
                videos = multimodal_content.get('videos', [])
                if videos:
                    content_types.append("video")
                    # Estimate video duration
                    estimated_duration = len(videos) * 30.0  # Assume 30s per file
                    media_usage.video_seconds = estimated_duration
                
                # Preprocessing overhead
                media_usage.preprocessing_ms = processing_time_ms * 0.1  # 10% overhead for multimodal
                
                breakdown.media_usage = media_usage
            
        except Exception as e:
            logger.warning(f"Error extracting Agno usage: {e}")
        
        # Set content types used
        breakdown.content_types = content_types
        
        return breakdown
    
    def aggregate_session_usage(self, usage_breakdowns: List[UsageBreakdown]) -> Dict[str, Any]:
        """Aggregate usage across multiple requests in a session."""
        if not usage_breakdowns:
            return {}
        
        total_tokens = 0
        total_processing_time = 0.0
        framework_distribution = {}
        model_distribution = {}
        framework_model_combinations = {}
        content_type_distribution = {}
        model_content_attribution = {}
        
        for breakdown in usage_breakdowns:
            total_tokens += breakdown.total_tokens
            total_processing_time += breakdown.processing_time_ms
            
            # Framework distribution
            framework_distribution[breakdown.framework] = framework_distribution.get(breakdown.framework, 0) + 1
            
            # Model distribution
            model_distribution[breakdown.model] = model_distribution.get(breakdown.model, 0) + 1
            
            # Framework + Model combinations for detailed tracking
            combo_key = f"{breakdown.framework}:{breakdown.model}"
            if combo_key not in framework_model_combinations:
                framework_model_combinations[combo_key] = {
                    "requests": 0,
                    "total_tokens": 0,
                    "avg_processing_time": 0.0
                }
            framework_model_combinations[combo_key]["requests"] += 1
            framework_model_combinations[combo_key]["total_tokens"] += breakdown.total_tokens
            framework_model_combinations[combo_key]["avg_processing_time"] += breakdown.processing_time_ms
            
            # Content type distribution
            for content_type in breakdown.content_types:
                content_type_distribution[content_type] = content_type_distribution.get(content_type, 0) + 1
                
                # Model + Content type attribution
                model_content_key = f"{breakdown.model}:{content_type}"
                if model_content_key not in model_content_attribution:
                    model_content_attribution[model_content_key] = {
                        "requests": 0,
                        "tokens": 0
                    }
                model_content_attribution[model_content_key]["requests"] += 1
                model_content_attribution[model_content_key]["tokens"] += breakdown.total_tokens
        
        # Calculate averages for framework-model combinations
        for combo_data in framework_model_combinations.values():
            if combo_data["requests"] > 0:
                combo_data["avg_processing_time"] = combo_data["avg_processing_time"] / combo_data["requests"]
                combo_data["avg_tokens_per_request"] = combo_data["total_tokens"] / combo_data["requests"]
        
        return {
            "session_summary": {
                "total_requests": len(usage_breakdowns),
                "total_tokens": total_tokens,
                "total_processing_time_ms": total_processing_time,
                "average_tokens_per_request": round(total_tokens / len(usage_breakdowns), 2),
                "framework_distribution": framework_distribution,
                "model_distribution": model_distribution,
                "content_type_distribution": content_type_distribution,
                "framework_model_combinations": {
                    k: {
                        "requests": v["requests"],
                        "total_tokens": v["total_tokens"],
                        "avg_processing_time_ms": round(v["avg_processing_time"], 2),
                        "avg_tokens_per_request": round(v["avg_tokens_per_request"], 2)
                    } for k, v in framework_model_combinations.items()
                },
                "model_content_attribution": model_content_attribution,
            },
            "detailed_breakdowns": [asdict(breakdown) for breakdown in usage_breakdowns]
        }
    
    def create_legacy_compatible_usage(self, breakdown: UsageBreakdown) -> Dict[str, Any]:
        """Create usage data compatible with existing database schema."""
        return {
            "framework": breakdown.framework,
            "model": breakdown.model,
            "request_tokens": breakdown.input_tokens,
            "response_tokens": breakdown.output_tokens,
            "total_tokens": breakdown.total_tokens,
            "cached_tokens": breakdown.cached_tokens,
            "processing_time_ms": breakdown.processing_time_ms,
            "content_types": breakdown.content_types,
            # Enhanced multimodal data
            "media_usage": asdict(breakdown.media_usage) if breakdown.media_usage else None,
            "framework_events": breakdown.framework_events,
            "request_timestamp": breakdown.request_timestamp,
        }


"""Factory for creating Claude Code SDK executor."""

import logging
import os
from pathlib import Path
from typing import Optional

from .executor_base import ExecutorBase

logger = logging.getLogger(__name__)


class ExecutorFactory:
    """Factory for creating SDK executor."""
    
    @staticmethod
    def create_executor(**kwargs) -> ExecutorBase:
        """Create SDK executor.
        
        Args:
            **kwargs: Additional arguments for the executor
            
        Returns:
            ClaudeSDKExecutor instance
        """
        logger.debug("Creating SDK executor")
        
        from .sdk_executor import ClaudeSDKExecutor
        
        # Create environment manager if needed
        environment_manager = None
        if kwargs.get('use_environment_manager', True):
            from .cli_environment import CLIEnvironmentManager
            
            workspace_base = kwargs.get('workspace_base', 
                                      os.environ.get("AUTOMAGIK_CLAUDE_LOCAL_WORKSPACE", "/tmp/claude-workspace"))
            
            environment_manager = CLIEnvironmentManager(
                base_path=Path(workspace_base)
            )
        
        return ClaudeSDKExecutor(environment_manager=environment_manager)


"""Common helpers for working with Evolution (WhatsApp-webhook) payloads.

This module re-exports the `EvolutionMessagePayload` model from the centralized
src.channels.models location, providing a stable import path for agents that
need to work with Evolution webhook payloads.

Example
-------
    from automagik.agents.common.evolution import EvolutionMessagePayload

    payload = EvolutionMessagePayload(**incoming_dict)
    user_number = payload.get_user_number()

This module provides backward compatibility while channeling imports to the
centralized channel models location.
"""

from __future__ import annotations

# Re-export the model from the centralized channels location. Keeping a local alias makes
# the public symbol independent of the module path, so callers only
# ever need to import *here*.
from automagik.channels.models import (
    EvolutionMessagePayload as _EvolutionMessagePayload,
)

# Public API
EvolutionMessagePayload = _EvolutionMessagePayload

__all__ = [
    "EvolutionMessagePayload",
] 


"""Enhanced Simple Agent using new framework patterns with multimodal support."""
from typing import Dict, List

from automagik.agents.models.automagik_agent import AutomagikAgent
from .prompts.prompt import AGENT_PROMPT

# Export commonly used functions for backward compatibility with tests
from automagik.agents.common.message_parser import (
    extract_all_messages,
    extract_tool_calls,
    extract_tool_outputs,
)


class SimpleAgent(AutomagikAgent):
    """Enhanced Simple Agent with multimodal capabilities.
    
    Features:
    - Image analysis and description
    - Document reading and summarization  
    - Audio transcription (when supported)
    - Automatic model switching to vision-capable models
    - Built-in multimodal analysis tools
    """
    
    def __init__(self, config: Dict[str, str]) -> None:
        """Initialize with complete multimodal setup."""
        # inject multimodal defaults
        if config is None:
            config = {}
        
        # Enhanced multimodal configuration
        config.setdefault("supported_media", ["image", "audio", "video", "document"])
        config.setdefault("auto_enhance_prompts", True)
        config.setdefault("enable_agno_for_multimodal", True)  # Use Agno for multimodal content
        # Use "auto" to enable framework switching based on content type
        config.setdefault("framework_type", "auto")

        super().__init__(config)

        self._code_prompt_text = AGENT_PROMPT

        # dependencies setup
        self.dependencies = self.create_default_dependencies()
        if self.db_id:
            self.dependencies.set_agent_id(self.db_id)

        # Register default tools
        self.tool_registry.register_default_tools(self.context)

        # Register Evolution WhatsApp helpers for parity with Sofia
        self.tool_registry.register_evolution_tools(self.context)

        # Register enhanced multimodal tools
        self._register_multimodal_tools()
        
        # Add multimodal capability tool
        self._register_media_capabilities_tool()

    def _register_multimodal_tools(self):
        """Register multimodal analysis tools using common helper."""
        try:
            from automagik.agents.common.multimodal_helper import register_multimodal_tools
            register_multimodal_tools(self.tool_registry, self.dependencies)
        except ImportError:
            # Fallback: register basic multimodal tools manually
            self._register_basic_multimodal_tools()
    
    def _register_basic_multimodal_tools(self):
        """Register basic multimodal tools as fallback."""
        
        async def analyze_image(ctx, description: str = "Analyze this image") -> str:
            """Analyze images attached to messages."""
            return f"Image analysis requested: {description}. The framework will automatically process any attached images."
            
        async def transcribe_audio(ctx, language: str = "auto") -> str:
            """Transcribe audio files."""
            return f"Audio transcription requested in language: {language}. The framework will automatically process any attached audio files."
            
        async def analyze_document(ctx, extract_type: str = "summary") -> str:
            """Analyze documents (PDFs, text files, etc)."""
            return f"Document analysis requested (type: {extract_type}). The framework will automatically process any attached documents."
        
        self.tool_registry.register_tool(analyze_image)
        self.tool_registry.register_tool(transcribe_audio)
        self.tool_registry.register_tool(analyze_document)
    
    def _register_media_capabilities_tool(self):
        """Register tool to describe multimodal capabilities."""
        
        async def describe_multimodal_capabilities(ctx) -> str:
            """Describe what media types and analysis I can perform."""
            return """🎯 **Enhanced Multimodal Capabilities**

📷 **Images**: 
- Object detection and recognition
- Scene description and analysis
- Text extraction (OCR) from images
- Chart and diagram interpretation

🎵 **Audio**:
- Speech transcription (multiple languages)
- Speaker identification
- Audio quality analysis
- Background sound detection

📄 **Documents**:
- PDF text extraction
- Document structure analysis
- Content summarization
- Table and data extraction

🎥 **Video** (limited):
- Frame extraction and analysis
- Content summarization

🤖 **Framework**: Automatically uses optimal framework (Agno for multimodal, PydanticAI for text)
⚡ **Performance**: Ultra-fast processing with comprehensive usage tracking

Simply attach any media files and I'll analyze them automatically!"""
        
        self.tool_registry.register_tool(describe_multimodal_capabilities)


def create_agent(config: Dict[str, str]) -> SimpleAgent:
    """Factory function to create enhanced simple agent with multimodal support."""
    try:
        return SimpleAgent(config)
    except Exception:
        from automagik.agents.models.placeholder import PlaceholderAgent
        return PlaceholderAgent(config)


"""
Loose Filtering Helper for Airtable

This module provides flexible, forgiving filtering strategies that work
even when users don't provide exact field names, character matches, etc.
"""

import logging
from typing import Dict, List, Optional
from automagik.tools.airtable.tool import list_records

logger = logging.getLogger(__name__)


async def loose_milestone_search(
    ctx: Dict,
    search_term: str,
    milestone_table_id: str,
    base_id: str
) -> Optional[str]:
    """
    Find a milestone using loose, flexible search strategies.
    
    Args:
        ctx: Runtime context
        search_term: User's search term (e.g., "Automagik - Plataforma")
        milestone_table_id: ID of the milestones table
        base_id: Airtable base ID
    
    Returns:
        Milestone record ID if found, None otherwise
    """
    
    # Extract key words from search term
    key_words = []
    for word in search_term.replace('-', ' ').replace('–', ' ').split():
        if len(word) > 2:  # Only use meaningful words
            key_words.append(word)
    
    logger.info(f"Searching for milestone with key words: {key_words}")
    
    # Strategy 1: Try partial search with each key word
    for word in key_words:
        field_names = ["Name", "Milestone Name", "Title"]
        
        for field_name in field_names:
            try:
                filter_formula = f"SEARCH('{word}', {{{field_name}}})"
                result = await list_records(
                    ctx,
                    table=milestone_table_id,
                    filter_formula=filter_formula,
                    base_id=base_id,
                    page_size=10
                )
                
                if result["success"] and result["records"]:
                    # Check if any result contains most of our key words
                    for record in result["records"]:
                        milestone_fields = record.get("fields", {})
                        milestone_name = (
                            milestone_fields.get("Name", "") + " " + 
                            milestone_fields.get("Milestone Name", "")
                        ).lower()
                        
                        # Count how many key words match
                        matches = sum(1 for kw in key_words if kw.lower() in milestone_name)
                        
                        if matches >= len(key_words) * 0.6:  # 60% match threshold
                            logger.info(f"Found milestone match: {record['id']} - {milestone_name}")
                            return record["id"]
                            
            except Exception as e:
                logger.debug(f"Search attempt failed for {word} in {field_name}: {e}")
                continue
    
    # Strategy 2: Get all milestones and do fuzzy matching
    try:
        all_milestones = await list_records(
            ctx,
            table=milestone_table_id,
            base_id=base_id,
            page_size=50
        )
        
        if all_milestones["success"]:
            search_term.lower()
            
            for record in all_milestones["records"]:
                milestone_fields = record.get("fields", {})
                milestone_name = (
                    milestone_fields.get("Name", "") + " " + 
                    milestone_fields.get("Milestone Name", "")
                ).lower()
                
                # Check for fuzzy matches
                if any(kw.lower() in milestone_name for kw in key_words):
                    logger.info(f"Fuzzy match found: {record['id']} - {milestone_name}")
                    return record["id"]
                    
    except Exception as e:
        logger.debug(f"Fuzzy search failed: {e}")
    
    logger.warning(f"No milestone found for search term: {search_term}")
    return None


async def loose_person_search(
    ctx: Dict,
    person_name: str,
    team_table_id: str,
    base_id: str
) -> Optional[str]:
    """
    Find a person using loose, flexible search strategies.
    
    Args:
        ctx: Runtime context
        person_name: Person's name (e.g., "Cezar Vasconcelos")
        team_table_id: ID of the team members table
        base_id: Airtable base ID
    
    Returns:
        Person record ID if found, None otherwise
    """
    
    # Extract name parts
    name_parts = person_name.split()
    
    # Strategy 1: Try searching with full name
    field_names = ["Name", "Full Name", "Team Member", "Display Name"]
    
    for field_name in field_names:
        try:
            filter_formula = f"SEARCH('{person_name}', {{{field_name}}})"
            result = await list_records(
                ctx,
                table=team_table_id,
                filter_formula=filter_formula,
                base_id=base_id,
                page_size=5
            )
            
            if result["success"] and result["records"]:
                return result["records"][0]["id"]
                
        except Exception as e:
            logger.debug(f"Full name search failed for {field_name}: {e}")
            continue
    
    # Strategy 2: Try searching with individual name parts
    for part in name_parts:
        if len(part) > 2:  # Skip short parts like initials
            for field_name in field_names:
                try:
                    filter_formula = f"SEARCH('{part}', {{{field_name}}})"
                    result = await list_records(
                        ctx,
                        table=team_table_id,
                        filter_formula=filter_formula,
                        base_id=base_id,
                        page_size=10
                    )
                    
                    if result["success"] and result["records"]:
                        # Check if any result is a good match
                        for record in result["records"]:
                            record_fields = record.get("fields", {})
                            record_name = (
                                record_fields.get("Name", "") + " " +
                                record_fields.get("Full Name", "")
                            ).lower()
                            
                            # If multiple name parts match, it's probably the right person
                            matches = sum(1 for np in name_parts if np.lower() in record_name)
                            if matches >= len(name_parts) * 0.7:  # 70% match
                                logger.info(f"Found person match: {record['id']} - {record_name}")
                                return record["id"]
                                
                except Exception as e:
                    logger.debug(f"Name part search failed for {part} in {field_name}: {e}")
                    continue
    
    logger.warning(f"No person found for: {person_name}")
    return None


async def create_loose_filter_for_milestone_and_person(
    ctx: Dict,
    person_name: str,
    milestone_name: str,
    base_id: str,
    additional_conditions: Optional[List[str]] = None
) -> Optional[str]:
    """
    Create a loose filter combining person and milestone searches.
    
    Args:
        ctx: Runtime context
        person_name: Person to search for
        milestone_name: Milestone to search for
        base_id: Airtable base ID
        additional_conditions: Other conditions to add
    
    Returns:
        Filter formula string or None if components can't be found
    """
    
    # Find table IDs
    from automagik.tools.airtable.tool import list_tables
    tables_result = await list_tables(ctx, base_id=base_id)
    
    if not tables_result["success"]:
        return None
    
    milestone_table_id = None
    team_table_id = None
    
    for table in tables_result["tables"]:
        table_name = table["name"].lower()
        if "milestone" in table_name:
            milestone_table_id = table["id"]
        elif "team" in table_name or "member" in table_name:
            team_table_id = table["id"]
    
    conditions = []
    
    # Try to find person
    if person_name and team_table_id:
        person_id = await loose_person_search(ctx, person_name, team_table_id, base_id)
        if person_id:
            # Use both ID and name search for maximum flexibility
            person_condition = f"OR(SEARCH('{person_name}', {{Assigned Team Members}}), SEARCH('{person_id}', {{Assigned Team Members}}))"
            conditions.append(person_condition)
        else:
            # Fallback to just name search
            conditions.append(f"SEARCH('{person_name}', {{Assigned Team Members}})")
    
    # Try to find milestone
    if milestone_name and milestone_table_id:
        milestone_id = await loose_milestone_search(ctx, milestone_name, milestone_table_id, base_id)
        if milestone_id:
            # Use both ID and name search for maximum flexibility
            milestone_condition = f"OR(SEARCH('{milestone_name}', {{Related Milestones}}), SEARCH('{milestone_id}', {{Related Milestones}}))"
            conditions.append(milestone_condition)
        else:
            # Fallback to just name search
            conditions.append(f"SEARCH('{milestone_name}', {{Related Milestones}})")
    
    # Add any additional conditions
    if additional_conditions:
        conditions.extend(additional_conditions)
    
    if conditions:
        if len(conditions) == 1:
            return conditions[0]
        else:
            return f"AND({', '.join(conditions)})"
    
    return None


async def loose_status_filter(status_term: str) -> str:
    """
    Create a loose status filter that handles variations.
    
    Args:
        status_term: Status to search for (e.g., "to do", "A fazer", "working")
    
    Returns:
        Filter formula string
    """
    
    # Map common variations to actual status values
    status_mapping = {
        "to do": "A fazer",
        "todo": "A fazer", 
        "pending": "A fazer",
        "working": "Estou trabalhando",
        "in progress": "Estou trabalhando",
        "blocked": "Estou bloqueado",
        "completed": "Terminei",
        "done": "Terminei",
        "finished": "Terminei"
    }
    
    status_lower = status_term.lower()
    
    # Check for exact mapping
    if status_lower in status_mapping:
        return f"{{Status}} = '{status_mapping[status_lower]}'"
    
    # Check for partial matches
    for key, value in status_mapping.items():
        if key in status_lower or status_lower in key:
            return f"{{Status}} = '{value}'"
    
    # Fallback: search within status field
    return f"SEARCH('{status_term}', {{Status}})" 


"""Placeholder agent implementation.

This module provides a basic placeholder agent to use when an agent fails to initialize.
"""

import logging
from typing import Dict, Any, Optional, Union

from automagik.agents.models.automagik_agent import AutomagikAgent
from automagik.agents.models.response import AgentResponse
from automagik.memory.message_history import MessageHistory

logger = logging.getLogger(__name__)

class PlaceholderAgent(AutomagikAgent):
    """A minimal placeholder agent for error cases."""
    
    def __init__(self, config: Dict[str, str]):
        """Initialize the placeholder agent.
        
        Args:
            config: Configuration dictionary
        """
        error_msg = config.get("error", "Unknown error")
        name = config.get("name", "placeholder_agent")
        
        # Initialize the base agent first
        super().__init__(config)
        
        # Set up prompt text to be registered later
        self._code_prompt_text = f"You are a placeholder agent named {name}. " \
                        f"The original agent failed to initialize with error: {error_msg}"
        self._prompt_registered = False
        
        self.error = error_msg
        self.name = name
        logger.warning(f"Created placeholder agent '{name}' with error: {error_msg}")
        
    async def run(self, input_text: str, *, multimodal_content=None, system_message=None, message_history_obj=None, channel_payload=None, message_limit=None) -> AgentResponse:
        """Run the placeholder agent.
        
        Args:
            input_text: Text input for the agent
            multimodal_content: Optional multimodal content
            system_message: Optional system message
            message_history_obj: Optional MessageHistory instance
            channel_payload: Optional channel-specific payload
            message_limit: Optional message limit
            
        Returns:
            AgentResponse with error message
        """
        # Try to register and load the prompt
        if self.db_id:
            await self._check_and_register_prompt()
            await self.load_active_prompt_template(status_key="default")
        
        message = f"I'm sorry, but the agent failed to initialize with error: {self.error}"
        
        return AgentResponse(
            text=message,
            success=False,
            error_message=self.error
        )
        
    async def process_message(self, user_message: Union[str, Dict[str, Any]], 
                              session_id: Optional[str] = None, 
                              agent_id: Optional[Union[int, str]] = None, 
                              user_id: int = 1, 
                              context: Optional[Dict] = None, 
                              message_history: Optional[MessageHistory] = None) -> AgentResponse:
        """Process a user message.
        
        Args:
            user_message: User message text or dictionary
            session_id: Optional session ID
            agent_id: Optional agent ID
            user_id: User ID (default 1)
            context: Optional context dictionary
            message_history: Optional MessageHistory instance
            
        Returns:
            AgentResponse with error message
        """
        if isinstance(user_message, dict):
            content = user_message.get("content", "")
        else:
            content = user_message
            
        return await self.run(content) 


"""Google Drive tools for Automagik Agents.

Provides tools for interacting with Google Drive via API.
"""

# Import from tool module
from automagik.tools.google_drive.tool import (
    search_files,
    get_file_content,
    get_search_files_description,
    get_file_content_description
)

# Import schema models
from automagik.tools.google_drive.schema import (
    GoogleDriveFile,
    SearchFilesResponse,
    GetFileContentResponse
)

# Import interface
from automagik.tools.google_drive.interface import (
    GoogleDriveTools,
    google_drive_tools
)

# Export public API
__all__ = [
    # Tool functions
    'search_files',
    'get_file_content',
    
    # Description functions
    'get_search_files_description',
    'get_file_content_description',
    
    # Schema models
    'GoogleDriveFile',
    'SearchFilesResponse',
    'GetFileContentResponse',
    
    # Interface
    'GoogleDriveTools',
    'google_drive_tools'
] 


import logging
from typing import Optional

from fastapi import HTTPException, status
from fastapi.concurrency import run_in_threadpool

from automagik.api.models import PromptResponse, PromptListResponse
from automagik.db.models import PromptCreate, PromptUpdate
from automagik.db.repository import prompt as prompt_repo

# Configure logger
logger = logging.getLogger(__name__)

async def list_prompts(agent_id: int, status_key: Optional[str] = None) -> PromptListResponse:
    """
    List all prompts for an agent, optionally filtered by status key.
    
    Args:
        agent_id: The agent ID
        status_key: Optional status key to filter by
        
    Returns:
        PromptListResponse with the list of prompts
    """
    try:
        prompts = await run_in_threadpool(prompt_repo.get_prompts_by_agent_id, agent_id, status_key)
        
        # Convert DB models to API response models
        prompt_responses = [
            PromptResponse(
                id=prompt.id,
                agent_id=prompt.agent_id,
                prompt_text=prompt.prompt_text,
                version=prompt.version,
                is_active=prompt.is_active,
                is_default_from_code=prompt.is_default_from_code,
                status_key=prompt.status_key,
                name=prompt.name,
                created_at=prompt.created_at,
                updated_at=prompt.updated_at
            )
            for prompt in prompts
        ]
        
        return PromptListResponse(
            prompts=prompt_responses,
            total=len(prompt_responses),
            agent_id=agent_id
        )
    except Exception as e:
        logger.error(f"Error listing prompts for agent {agent_id}: {str(e)}")
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed to list prompts: {str(e)}"
        )

async def get_prompt(prompt_id: int) -> PromptResponse:
    """
    Get a prompt by ID.
    
    Args:
        prompt_id: The prompt ID
        
    Returns:
        PromptResponse with the prompt details
    """
    try:
        prompt = await run_in_threadpool(prompt_repo.get_prompt_by_id, prompt_id)
        
        if not prompt:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail=f"Prompt with ID {prompt_id} not found"
            )
        
        return PromptResponse(
            id=prompt.id,
            agent_id=prompt.agent_id,
            prompt_text=prompt.prompt_text,
            version=prompt.version,
            is_active=prompt.is_active,
            is_default_from_code=prompt.is_default_from_code,
            status_key=prompt.status_key,
            name=prompt.name,
            created_at=prompt.created_at,
            updated_at=prompt.updated_at
        )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Error getting prompt {prompt_id}: {str(e)}")
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed to get prompt: {str(e)}"
        )

async def create_prompt(agent_id: int, prompt_data: dict) -> PromptResponse:
    """
    Create a new prompt for an agent.
    
    Args:
        agent_id: The agent ID
        prompt_data: The prompt data
        
    Returns:
        PromptResponse with the created prompt details
    """
    try:
        # Create a PromptCreate object
        create_data = PromptCreate(
            agent_id=agent_id,
            prompt_text=prompt_data["prompt_text"],
            status_key=prompt_data.get("status_key", "default"),
            name=prompt_data.get("name"),
            is_active=prompt_data.get("is_active", False),
            version=prompt_data.get("version", 1),
            is_default_from_code=False  # API-created prompts are never default from code
        )
        
        # Create the prompt
        prompt_id = await run_in_threadpool(prompt_repo.create_prompt, create_data)
        
        if not prompt_id:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail="Failed to create prompt"
            )
        
        # Get the created prompt
        prompt = await run_in_threadpool(prompt_repo.get_prompt_by_id, prompt_id)
        
        return PromptResponse(
            id=prompt.id,
            agent_id=prompt.agent_id,
            prompt_text=prompt.prompt_text,
            version=prompt.version,
            is_active=prompt.is_active,
            is_default_from_code=prompt.is_default_from_code,
            status_key=prompt.status_key,
            name=prompt.name,
            created_at=prompt.created_at,
            updated_at=prompt.updated_at
        )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Error creating prompt for agent {agent_id}: {str(e)}")
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed to create prompt: {str(e)}"
        )

async def update_prompt(prompt_id: int, prompt_data: dict) -> PromptResponse:
    """
    Update an existing prompt.
    
    Args:
        prompt_id: The prompt ID
        prompt_data: The updated prompt data
        
    Returns:
        PromptResponse with the updated prompt details
    """
    try:
        # Get the existing prompt
        existing_prompt = await run_in_threadpool(prompt_repo.get_prompt_by_id, prompt_id)
        
        if not existing_prompt:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail=f"Prompt with ID {prompt_id} not found"
            )
        
        # Create an update object
        update_data = PromptUpdate(
            prompt_text=prompt_data.get("prompt_text"),
            name=prompt_data.get("name"),
            is_active=prompt_data.get("is_active")
        )
        
        # Update the prompt
        success = await run_in_threadpool(prompt_repo.update_prompt, prompt_id, update_data)
        
        if not success:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail="Failed to update prompt"
            )
        
        # Get the updated prompt
        updated_prompt = await run_in_threadpool(prompt_repo.get_prompt_by_id, prompt_id)
        
        return PromptResponse(
            id=updated_prompt.id,
            agent_id=updated_prompt.agent_id,
            prompt_text=updated_prompt.prompt_text,
            version=updated_prompt.version,
            is_active=updated_prompt.is_active,
            is_default_from_code=updated_prompt.is_default_from_code,
            status_key=updated_prompt.status_key,
            name=updated_prompt.name,
            created_at=updated_prompt.created_at,
            updated_at=updated_prompt.updated_at
        )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Error updating prompt {prompt_id}: {str(e)}")
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed to update prompt: {str(e)}"
        )

async def set_prompt_active(prompt_id: int, is_active: bool = True) -> PromptResponse:
    """
    Set a prompt as active or inactive.
    
    Args:
        prompt_id: The prompt ID
        is_active: Whether to set as active
        
    Returns:
        PromptResponse with the updated prompt details
    """
    try:
        # Get the existing prompt
        existing_prompt = await run_in_threadpool(prompt_repo.get_prompt_by_id, prompt_id)
        
        if not existing_prompt:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail=f"Prompt with ID {prompt_id} not found"
            )
        
        # Set the prompt active status
        success = await run_in_threadpool(prompt_repo.set_prompt_active, prompt_id, is_active)
        
        if not success:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=f"Failed to set prompt {prompt_id} active status to {is_active}"
            )
        
        # Get the updated prompt
        updated_prompt = await run_in_threadpool(prompt_repo.get_prompt_by_id, prompt_id)
        
        return PromptResponse(
            id=updated_prompt.id,
            agent_id=updated_prompt.agent_id,
            prompt_text=updated_prompt.prompt_text,
            version=updated_prompt.version,
            is_active=updated_prompt.is_active,
            is_default_from_code=updated_prompt.is_default_from_code,
            status_key=updated_prompt.status_key,
            name=updated_prompt.name,
            created_at=updated_prompt.created_at,
            updated_at=updated_prompt.updated_at
        )
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Error setting prompt {prompt_id} active status to {is_active}: {str(e)}")
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed to set prompt active status: {str(e)}"
        )

async def delete_prompt(prompt_id: int) -> dict:
    """
    Delete a prompt.
    
    Args:
        prompt_id: The prompt ID
        
    Returns:
        Success message
    """
    try:
        # Get the existing prompt
        existing_prompt = await run_in_threadpool(prompt_repo.get_prompt_by_id, prompt_id)
        
        if not existing_prompt:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail=f"Prompt with ID {prompt_id} not found"
            )
        
        # Delete the prompt
        success = await run_in_threadpool(prompt_repo.delete_prompt, prompt_id)
        
        if not success:
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=f"Failed to delete prompt {prompt_id}"
            )
        
        return {
            "status": "success",
            "detail": f"Prompt with ID {prompt_id} deleted successfully"
        }
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Error deleting prompt {prompt_id}: {str(e)}")
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed to delete prompt: {str(e)}"
        ) 


"""PydanticAI framework integration for AutomagikAgent."""
import logging
from typing import Dict, List, Optional, Any, Union, Type

from automagik.agents.models.ai_frameworks.base import AgentAIFramework, AgentConfig
from automagik.agents.models.response import AgentResponse
from automagik.agents.models.dependencies import BaseDependencies

logger = logging.getLogger(__name__)


class PydanticAIFramework(AgentAIFramework):
    """PydanticAI framework adapter for AutomagikAgent."""
    
    def __init__(self, config: AgentConfig):
        """Initialize the PydanticAI framework adapter."""
        super().__init__(config)
        self._tools = []
        self._dependencies_type = None
        
    async def initialize(self, 
                        tools: List[Any], 
                        dependencies_type: Type[BaseDependencies],
                        mcp_servers: Optional[List[Any]] = None,
                        system_prompt: Optional[str] = None) -> None:
        """Initialize the PydanticAI agent instance."""
        try:
            from pydantic_ai import Agent
            
            # Store tools and dependencies type
            self._tools = tools or []
            self._dependencies_type = dependencies_type
            
            # Convert tools to PydanticAI format
            converted_tools = self.convert_tools(self._tools)
            
            # Create PydanticAI agent (without tools initially)
            # Use the provided system prompt or a default
            actual_system_prompt = system_prompt or "You are a helpful assistant."
            logger.debug(f"Creating PydanticAI agent with system prompt: {actual_system_prompt[:100]}...")
            
            self._agent_instance = Agent(
                model=self.config.model,
                deps_type=dependencies_type,
                retries=self.config.retries,
                output_type=str,  # Default to string result (updated API)
                system_prompt=actual_system_prompt
            )
            
            # Register converted tools using the decorator approach
            for tool in converted_tools:
                if callable(tool) and hasattr(tool, '__name__'):
                    # Use the tool decorator to register the function
                    self._agent_instance.tool(tool)
                    logger.debug(f"Registered tool: {tool.__name__}")
                    
            self.is_initialized = True
            logger.info(f"PydanticAI agent initialized with {len(converted_tools)} tools")
            
        except Exception as e:
            logger.error(f"Error initializing PydanticAI framework: {e}")
            self.is_initialized = False
            raise
    
    async def run(self,
                  user_input: Union[str, List[Any]],
                  dependencies: BaseDependencies,
                  message_history: Optional[List[Any]] = None,
                  system_prompt: Optional[str] = None,
                  **kwargs) -> AgentResponse:
        """Run the PydanticAI agent."""
        if not self.is_ready:
            raise RuntimeError("PydanticAI framework not initialized")
            
        try:
            import time
            start_time = time.time()
            
            # Detect multimodal content from user input
            multimodal_content = self._detect_multimodal_content(user_input, kwargs)
            
            # Format message history for PydanticAI
            formatted_history = self.format_message_history(message_history or [])
            if formatted_history:
                logger.info(f"📚 PydanticAI: Formatted {len(formatted_history)} messages from history")
            else:
                logger.debug("📭 PydanticAI: No message history to format")
            
            # Update the agent's system prompt if provided
            if system_prompt:
                # Try to update the internal system prompt dynamically
                # This is a workaround since PydanticAI sets system prompt at init time
                logger.debug(f"Attempting to update system prompt. Agent instance attributes: {[attr for attr in dir(self._agent_instance) if not attr.startswith('__')]}")
                
                # Check various possible attribute names
                updated = False
                for attr_name in ['_system_prompt', 'system_prompt', '_system', 'system']:
                    if hasattr(self._agent_instance, attr_name):
                        try:
                            setattr(self._agent_instance, attr_name, system_prompt)
                            logger.info(f"Successfully updated PydanticAI {attr_name} to: {system_prompt[:100]}...")
                            updated = True
                            break
                        except Exception as e:
                            logger.debug(f"Could not update {attr_name}: {e}")
                
                if not updated:
                    logger.warning("Could not update PydanticAI system prompt - no writable attribute found")
                
                # Also add system prompt to message history
                from pydantic_ai.messages import ModelRequest, SystemPromptPart
                
                # Check if there's already a system message in the history
                has_system_message = False
                for msg in formatted_history:
                    if hasattr(msg, 'parts') and any(isinstance(part, SystemPromptPart) for part in msg.parts):
                        has_system_message = True
                        break
                
                # Only add system prompt if there isn't one already
                if not has_system_message:
                    system_message = ModelRequest(parts=[SystemPromptPart(content=system_prompt)])
                    # Insert system message at the beginning of history
                    formatted_history.insert(0, system_message)
                    logger.debug(f"Added system prompt to message history: {system_prompt[:100]}...")
            
            # Run the agent
            logger.info(f"🚀 PydanticAI: Running agent with {len(formatted_history)} messages in history")
            result = await self._agent_instance.run(
                user_input,
                deps=dependencies,
                message_history=formatted_history,
                **kwargs
            )
            
            # Calculate processing time
            processing_time_ms = (time.time() - start_time) * 1000
            
            # Extract tool information
            tool_calls = self.extract_tool_calls(result)
            tool_outputs = self.extract_tool_outputs(result)
            
            # Extract usage information with processing time and multimodal content
            usage_info = self.extract_usage_info(
                result, 
                processing_time_ms=processing_time_ms,
                multimodal_content=multimodal_content
            )
            
            # Create response (using updated API)
            response = AgentResponse(
                text=result.output if hasattr(result, 'output') else str(result),
                success=True,
                tool_calls=tool_calls,
                tool_outputs=tool_outputs,
                system_prompt=system_prompt,
                usage=usage_info
            )
            
            return response
            
        except Exception as e:
            logger.error(f"Error running PydanticAI agent: {e}")
            import traceback
            logger.error(f"Full traceback: {traceback.format_exc()}")
            
            # Send error notification
            from automagik.utils.error_notifications import notify_agent_error
            import asyncio
            asyncio.create_task(notify_agent_error(
                error=e,
                agent_name=getattr(dependencies, 'agent_name', 'unknown'),
                user_id=str(getattr(dependencies, 'user_id', None)),
                session_id=str(getattr(dependencies, 'session_id', None)),
                context={"framework": "pydantic_ai", "model": self.config.model}
            ))
            
            # Return user-friendly message
            return AgentResponse(
                text="I apologize, but I encountered an issue processing your request. Please try again in a moment. If the problem persists, our team has been notified and is working on it.",
                success=False,
                error_message=str(e)
            )
    
    def format_message_history(self, 
                              raw_messages: List[Any]) -> List[Any]:
        """Convert message history to PydanticAI format."""
        try:
            from pydantic_ai.messages import ModelRequest, ModelResponse, SystemPromptPart, UserPromptPart, TextPart
            
            formatted_messages = []
            
            for message in raw_messages:
                # Check if message is already a PydanticAI ModelMessage
                if hasattr(message, 'parts'):
                    # Already a PydanticAI message, use as-is
                    formatted_messages.append(message)
                    continue
                
                # Handle dictionary messages (legacy format)
                if isinstance(message, dict):
                    role = message.get('role', 'user')
                    content = message.get('content', '')
                    
                    if role == 'system':
                        # System messages are handled as ModelRequest with SystemPromptPart
                        formatted_messages.append(ModelRequest(parts=[SystemPromptPart(content=content)]))
                    elif role == 'assistant':
                        # Assistant messages are ModelResponse with TextPart
                        formatted_messages.append(ModelResponse(parts=[TextPart(content=content)]))
                    else:  # user
                        # User messages are ModelRequest with UserPromptPart
                        formatted_messages.append(ModelRequest(parts=[UserPromptPart(content=content)]))
                else:
                    # Unknown message type, log warning and skip
                    logger.warning(f"Unknown message type in history: {type(message)}")
                    
            return formatted_messages
            
        except Exception as e:
            logger.error(f"Error formatting message history: {e}")
            import traceback
            logger.error(f"Traceback: {traceback.format_exc()}")
            return []
    
    def extract_tool_calls(self, result: Any) -> List[Dict[str, Any]]:
        """Extract tool calls from PydanticAI result."""
        tool_calls = []
        tool_results = {}  # Map tool call IDs to results
        
        try:
            if hasattr(result, 'all_messages'):
                # First pass: collect tool results from ToolReturnPart
                for message in result.all_messages():
                    if hasattr(message, 'parts') and message.parts:
                        for part in message.parts:
                            # Check if this is a tool return part
                            if hasattr(part, 'tool_call_id') and hasattr(part, 'content') and 'ToolReturnPart' in str(type(part)):
                                tool_results[part.tool_call_id] = part.content
                
                # Second pass: extract tool calls from ToolCallPart and match with results
                for message in result.all_messages():
                    if hasattr(message, 'parts') and message.parts:
                        for part in message.parts:
                            # Check if this is a tool call part
                            if hasattr(part, 'tool_call_id') and hasattr(part, 'tool_name') and hasattr(part, 'args') and 'ToolCallPart' in str(type(part)):
                                tool_calls.append({
                                    'name': part.tool_name,
                                    'args': part.args,
                                    'id': part.tool_call_id,
                                    'result': tool_results.get(part.tool_call_id, None)
                                })
                            
        except Exception as e:
            logger.error(f"Error extracting tool calls: {e}")
            
        return tool_calls
    
    def extract_tool_outputs(self, result: Any) -> List[Dict[str, Any]]:
        """Extract tool outputs from PydanticAI result."""
        tool_outputs = []
        
        try:
            if hasattr(result, 'all_messages'):
                for message in result.all_messages():
                    if hasattr(message, 'content') and hasattr(message, 'tool_call_id'):
                        if message.tool_call_id:  # This is a tool response
                            tool_outputs.append({
                                'tool_call_id': message.tool_call_id,
                                'output': message.content
                            })
                            
        except Exception as e:
            logger.error(f"Error extracting tool outputs: {e}")
            
        return tool_outputs
    
    def extract_usage_info(self, result: Any, processing_time_ms: float = 0.0, multimodal_content: Optional[Dict[str, Any]] = None) -> Optional[Dict[str, Any]]:
        """Extract usage information from PydanticAI result using simplified usage calculator."""
        if not result:
            return None
        
        try:
            # Import our simplified usage calculator
            from automagik.utils.usage_calculator import UnifiedUsageCalculator
            calculator = UnifiedUsageCalculator()
            
            # Extract usage using our new simplified calculator
            breakdown = calculator.extract_pydantic_ai_usage(
                result=result,
                model=self.config.model,
                processing_time_ms=processing_time_ms,
                multimodal_content=multimodal_content
            )
            
            # Detect content types based on input
            content_types = ["text"]  # Always include text
            if multimodal_content:
                if multimodal_content.get("images"):
                    content_types.append("image")
                if multimodal_content.get("audio"):
                    content_types.append("audio")
                if multimodal_content.get("videos"):
                    content_types.append("video")
            
            breakdown.content_types = content_types
            
            # Convert to legacy compatible format for API response
            return calculator.create_legacy_compatible_usage(breakdown)
            
        except Exception as e:
            logger.error(f"Error extracting usage info: {e}")
            # Fallback to basic extraction
            try:
                # Detect content types for fallback usage (safely handle multimodal_content)
                content_types = ["text"]
                if multimodal_content and isinstance(multimodal_content, dict):
                    if multimodal_content.get("images"):
                        content_types.append("image")
                    if multimodal_content.get("audio"):
                        content_types.append("audio")
                    if multimodal_content.get("videos"):
                        content_types.append("video")
                
                usage_info = {
                    "framework": "pydantic_ai",
                    "model": self.config.model,
                    "request_tokens": 0,
                    "response_tokens": 0,
                    "total_tokens": 0,
                    "content_types": content_types,
                    "processing_time_ms": processing_time_ms
                }
                
                # Extract basic usage from result
                if hasattr(result, 'all_messages'):
                    for message in result.all_messages():
                        if hasattr(message, 'usage') and message.usage:
                            usage = message.usage
                            usage_info["request_tokens"] = (usage_info.get("request_tokens", 0) or 0) + (usage.request_tokens or 0)
                            usage_info["response_tokens"] = (usage_info.get("response_tokens", 0) or 0) + (usage.response_tokens or 0)
                            usage_info["total_tokens"] = (usage_info.get("total_tokens", 0) or 0) + (usage.total_tokens or 0)
                
                return usage_info if usage_info["total_tokens"] > 0 else None
                
            except Exception as fallback_error:
                logger.error(f"Fallback usage extraction failed: {fallback_error}")
                return None
    
    def _detect_multimodal_content(self, user_input: Union[str, List[Any]], kwargs: Dict[str, Any]) -> Optional[Dict[str, Any]]:
        """Detect multimodal content from user input and kwargs."""
        multimodal_content = {"images": [], "audio": [], "videos": []}
        has_multimodal = False
        
        try:
            # Check if user_input is a list (multimodal input)
            if isinstance(user_input, list):
                for item in user_input:
                    if isinstance(item, dict):
                        content_type = item.get("type", "").lower()
                        if "image" in content_type:
                            multimodal_content["images"].append(item)
                            has_multimodal = True
                        elif "audio" in content_type:
                            multimodal_content["audio"].append(item)
                            has_multimodal = True
                        elif "video" in content_type:
                            multimodal_content["videos"].append(item)
                            has_multimodal = True
            
            # Check kwargs for media content
            if "media_contents" in kwargs:
                media_contents = kwargs["media_contents"]
                if isinstance(media_contents, list):
                    for media in media_contents:
                        if isinstance(media, dict):
                            mime_type = media.get("mime_type", "").lower()
                            if "image" in mime_type:
                                multimodal_content["images"].append(media)
                                has_multimodal = True
                            elif "audio" in mime_type:
                                multimodal_content["audio"].append(media)
                                has_multimodal = True
                            elif "video" in mime_type:
                                multimodal_content["videos"].append(media)
                                has_multimodal = True
            
            # Check for image attachments in user_input string
            if isinstance(user_input, str):
                # Look for base64 image data or image URLs
                if "data:image" in user_input or "base64" in user_input.lower():
                    multimodal_content["images"].append({"type": "image", "detected": "base64_in_text"})
                    has_multimodal = True
            
            return multimodal_content if has_multimodal else None
            
        except Exception as e:
            logger.error(f"Error detecting multimodal content: {e}")
            return None
    
    def convert_tools(self, tools: List[Any]) -> List[Any]:
        """Convert tools to PydanticAI format."""
        converted_tools = []
        
        for tool in tools:
            try:
                # If it's already a function, use it directly
                if callable(tool):
                    converted_tools.append(tool)
                elif hasattr(tool, 'func') and callable(tool.func):
                    # If it's a wrapped tool, extract the function
                    converted_tools.append(tool.func)
                else:
                    logger.warning(f"Unable to convert tool: {tool}")
                    
            except Exception as e:
                logger.error(f"Error converting tool {tool}: {e}")
                
        return converted_tools
    
    async def cleanup(self) -> None:
        """Clean up PydanticAI resources."""
        self._agent_instance = None
        self.is_initialized = False
        logger.debug("PydanticAI framework cleaned up")


"""Database provider factory for selecting between PostgreSQL and SQLite."""

import logging
import os
from typing import Optional

from .base import DatabaseProvider
from .postgresql import PostgreSQLProvider
from .sqlite import SQLiteProvider

logger = logging.getLogger(__name__)

# Global provider instance
_provider: Optional[DatabaseProvider] = None

def get_database_provider() -> DatabaseProvider:
    """Get the configured database provider singleton."""
    global _provider
    
    if _provider is None:
        _provider = create_database_provider()
    
    return _provider

def create_database_provider() -> DatabaseProvider:
    """Create a new database provider based on configuration."""
    # Get database type from environment variable
    db_type = os.environ.get("AUTOMAGIK_DATABASE_TYPE", "sqlite").lower()
    
    # Only auto-detect PostgreSQL if AUTOMAGIK_DATABASE_TYPE is not explicitly set
    if "AUTOMAGIK_DATABASE_TYPE" not in os.environ:
        # Check for PostgreSQL connection string to auto-detect
        database_url = os.environ.get("AUTOMAGIK_DATABASE_URL", "")
        if database_url.startswith("postgresql://") or database_url.startswith("postgres://"):
            logger.info("PostgreSQL AUTOMAGIK_DATABASE_URL detected, auto-setting AUTOMAGIK_DATABASE_TYPE to postgresql")
            db_type = "postgresql"
    
    # Create appropriate provider
    if db_type == "postgresql":
        logger.info("Using PostgreSQL database provider")
        return PostgreSQLProvider()
    elif db_type == "sqlite":
        # Get SQLite database path
        sqlite_path = os.environ.get("AUTOMAGIK_SQLITE_DATABASE_PATH")
        logger.info(f"Using SQLite database provider (path: {sqlite_path or 'default'})")
        return SQLiteProvider(database_path=sqlite_path)
    else:
        raise ValueError(f"Unsupported database type: {db_type}. Supported types: postgresql, sqlite")

def reset_database_provider():
    """Reset the global provider instance. Useful for testing."""
    global _provider
    if _provider:
        try:
            _provider.close_connection_pool()
        except Exception as e:
            logger.warning(f"Error closing database provider: {e}")
    _provider = None

def get_supported_database_types() -> list[str]:
    """Get list of supported database types."""
    return ["postgresql", "sqlite"]

def is_database_type_supported(db_type: str) -> bool:
    """Check if a database type is supported."""
    return db_type.lower() in get_supported_database_types()

def get_database_type() -> str:
    """Get the currently configured database type."""
    provider = get_database_provider()
    return provider.get_database_type()

def supports_feature(feature: str) -> bool:
    """Check if the current database provider supports a specific feature."""
    provider = get_database_provider()
    return provider.supports_feature(feature)


"""Tool Wrapper Factory for PydanticAI agents.

This module provides utilities to eliminate duplication in tool wrapper creation
across PydanticAI agents, reducing boilerplate code significantly.
"""
import logging
from typing import Any, Dict, Callable, Optional, List
from functools import wraps
import inspect

logger = logging.getLogger(__name__)


class ToolWrapperFactory:
    """Factory for creating standardized tool wrappers for PydanticAI agents."""
    
    @staticmethod
    def create_context_wrapper(
        func: Callable,
        context: Dict[str, Any],
        preserve_signature: bool = True
    ) -> Callable:
        """Create a context-aware tool wrapper.
        
        Args:
            func: The function to wrap
            context: The agent context to inject
            preserve_signature: Whether to preserve the original function signature
            
        Returns:
            Wrapped function that injects context as first argument
        """
        if preserve_signature:
            # Preserve original signature but inject context
            sig = inspect.signature(func)
            
            @wraps(func)
            async def wrapper(ctx, *args, **kwargs):
                # Inject context as first argument after ctx
                return await func(context, *args, **kwargs)
                
            # Update wrapper signature to match original (minus context param)
            wrapper.__signature__ = sig
        else:
            # Simple wrapper without signature preservation
            async def wrapper(ctx, *args, **kwargs):
                return await func(context, *args, **kwargs)
            
            wrapper.__name__ = func.__name__
            wrapper.__doc__ = func.__doc__
        
        return wrapper
    
    @staticmethod
    def create_agent_tool_wrapper(
        agent_func: Callable,
        agent_context: Dict[str, Any],
        input_param: str = "input_text"
    ) -> Callable:
        """Create a wrapper for sub-agent calls.
        
        Args:
            agent_func: The agent function to wrap
            agent_context: The parent agent context
            input_param: Name of the input parameter
            
        Returns:
            Wrapped agent function
        """
        async def wrapper(ctx, **kwargs):
            # Ensure evolution_payload is available in context
            if hasattr(ctx, 'deps') and ctx.deps:
                ctx.deps.set_context(agent_context)
            
            # Extract the input text from kwargs
            input_text = kwargs.get(input_param, "")
            return await agent_func(ctx, input_text)
        
        wrapper.__name__ = agent_func.__name__
        wrapper.__doc__ = agent_func.__doc__
        return wrapper
    
    @staticmethod
    def create_batch_wrappers(
        functions: List[Callable],
        context: Dict[str, Any]
    ) -> Dict[str, Callable]:
        """Create multiple tool wrappers at once.
        
        Args:
            functions: List of functions to wrap
            context: The agent context to inject
            
        Returns:
            Dictionary mapping function names to wrapped functions
        """
        wrappers = {}
        for func in functions:
            wrapper = ToolWrapperFactory.create_context_wrapper(func, context)
            wrappers[func.__name__] = wrapper
        return wrappers
    
    @staticmethod
    def create_api_tool_wrapper(
        api_func: Callable,
        context: Dict[str, Any],
        response_formatter: Optional[Callable] = None
    ) -> Callable:
        """Create a wrapper for API integration tools.
        
        Args:
            api_func: The API function to wrap
            context: The agent context
            response_formatter: Optional function to format the response
            
        Returns:
            Wrapped API function with error handling
        """
        async def wrapper(ctx, **kwargs):
            try:
                result = await api_func(context, **kwargs)
                
                if response_formatter:
                    return response_formatter(result)
                return result
                
            except Exception as e:
                logger.error(f"Error in API tool {api_func.__name__}: {str(e)}")
                return {
                    "error": True,
                    "message": f"Failed to execute {api_func.__name__}: {str(e)}"
                }
        
        wrapper.__name__ = api_func.__name__
        wrapper.__doc__ = api_func.__doc__
        return wrapper
    
    @staticmethod
    def register_tools_from_module(
        tool_registry,
        module,
        context: Dict[str, Any],
        function_filter: Optional[Callable[[str], bool]] = None
    ) -> List[str]:
        """Auto-register all functions from a module as tools.
        
        Args:
            tool_registry: The agent's tool registry
            module: Module containing tool functions
            context: Agent context to inject
            function_filter: Optional filter for function names
            
        Returns:
            List of registered tool names
        """
        registered_tools = []
        
        for name in dir(module):
            if function_filter and not function_filter(name):
                continue
                
            attr = getattr(module, name)
            if callable(attr) and not name.startswith('_'):
                # Create wrapper for the function
                wrapper = ToolWrapperFactory.create_context_wrapper(attr, context)
                tool_registry.register_tool(wrapper)
                registered_tools.append(name)
                logger.debug(f"Auto-registered tool: {name}")
        
        return registered_tools


class ToolRegistrationHelper:
    """Helper class for common tool registration patterns."""
    
    @staticmethod
    def register_evolution_tools(tool_registry, context: Dict[str, Any]) -> None:
        """Register standard Evolution/WhatsApp tools."""
        tool_registry.register_evolution_tools(context)
    
    @staticmethod
    def register_blackpearl_tools(
        tool_registry,
        context: Dict[str, Any],
        tools: Optional[List[str]] = None
    ) -> None:
        """Register BlackPearl integration tools.
        
        Args:
            tool_registry: The agent's tool registry
            context: Agent context
            tools: Optional list of specific tools to register (default: all)
        """
        from automagik.tools.blackpearl import verificar_cnpj
        
        # Standard BlackPearl tools
        blackpearl_tools = {
            'verificar_cnpj': verificar_cnpj
        }
        
        tools_to_register = tools or blackpearl_tools.keys()
        
        for tool_name in tools_to_register:
            if tool_name in blackpearl_tools:
                wrapper = ToolWrapperFactory.create_context_wrapper(
                    blackpearl_tools[tool_name], 
                    context
                )
                tool_registry.register_tool(wrapper)
                logger.debug(f"Registered BlackPearl tool: {tool_name}")
    
    @staticmethod
    def register_specialized_agents(
        tool_registry,
        context: Dict[str, Any],
        agents: Dict[str, Callable]
    ) -> None:
        """Register specialized sub-agents as tools.
        
        Args:
            tool_registry: The agent's tool registry
            context: Agent context
            agents: Dictionary mapping agent names to agent functions
        """
        for agent_name, agent_func in agents.items():
            wrapper = ToolWrapperFactory.create_agent_tool_wrapper(
                agent_func, 
                context
            )
            tool_registry.register_tool(wrapper)
            logger.debug(f"Registered specialized agent: {agent_name}")


"""Configuration management for the tracing system."""

import os
from typing import List, Optional
from dataclasses import dataclass, field


@dataclass
class TracingConfig:
    """Configuration for tracing systems."""
    
    # Observability settings
    observability_enabled: bool = True
    observability_providers: List[str] = field(default_factory=list)
    
    # Telemetry settings  
    telemetry_enabled: bool = True
    telemetry_endpoint: str = "https://telemetry.namastex.ai/v1/traces"
    telemetry_anonymous_id: Optional[str] = None
    
    # Privacy settings
    disable_all_tracing: bool = False
    disable_in_ci: bool = True
    
    # Performance settings
    max_queue_size: int = 10000
    batch_size: int = 50
    batch_timeout_ms: int = 5000  # 5 seconds
    max_workers: int = 2
    
    # Sampling settings
    default_sampling_rate: float = 0.1  # 10%
    error_sampling_rate: float = 1.0    # 100%
    slow_threshold_ms: float = 1000     # 1 second
    
    @classmethod
    def from_env(cls) -> 'TracingConfig':
        """Load configuration from environment variables."""
        config = cls()
        
        # Global disable
        if os.getenv("AUTOMAGIK_DISABLE_ALL_TRACING", "false").lower() == "true":
            config.disable_all_tracing = True
            config.observability_enabled = False
            config.telemetry_enabled = False
            return config
        
        # Check for opt-out file
        opt_out_file = os.path.expanduser("~/.automagik-no-tracing")
        if os.path.exists(opt_out_file):
            config.disable_all_tracing = True
            config.observability_enabled = False
            config.telemetry_enabled = False
            return config
            
        # Observability configuration
        config.observability_enabled = os.getenv(
            "AUTOMAGIK_OBSERVABILITY_ENABLED", "true"
        ).lower() == "true"
        
        # Detect available providers based on API keys
        if config.observability_enabled:
            providers = []
            langwatch_key = os.getenv("LANGWATCH_API_KEY")
            if langwatch_key:
                providers.append("langwatch")
                print(f"🔑 LangWatch API key detected: {langwatch_key[:10]}...")
            if os.getenv("LANGFUSE_PUBLIC_KEY"):
                providers.append("langfuse")
            if os.getenv("LANGSMITH_API_KEY"):
                providers.append("langsmith")
            config.observability_providers = providers
            print(f"📊 Observability providers configured: {providers}")
        
        # Telemetry configuration
        config.telemetry_enabled = os.getenv(
            "AUTOMAGIK_TELEMETRY_ENABLED", "true"
        ).lower() == "true"
        
        if os.getenv("AUTOMAGIK_TELEMETRY_ENDPOINT"):
            config.telemetry_endpoint = os.getenv("AUTOMAGIK_TELEMETRY_ENDPOINT")
        
        # Auto-disable in CI unless explicitly enabled
        if any(os.getenv(var) for var in ["CI", "GITHUB_ACTIONS", "TRAVIS", "JENKINS"]):
            if os.getenv("AUTOMAGIK_ENABLE_TRACING_IN_CI", "false").lower() != "true":
                config.disable_in_ci = True
                config.observability_enabled = False
                config.telemetry_enabled = False
        
        # Performance settings from env
        if os.getenv("AUTOMAGIK_TRACE_QUEUE_SIZE"):
            config.max_queue_size = int(os.getenv("AUTOMAGIK_TRACE_QUEUE_SIZE"))
        if os.getenv("AUTOMAGIK_TRACE_BATCH_SIZE"):
            config.batch_size = int(os.getenv("AUTOMAGIK_TRACE_BATCH_SIZE"))
        if os.getenv("AUTOMAGIK_TRACE_WORKERS"):
            config.max_workers = int(os.getenv("AUTOMAGIK_TRACE_WORKERS"))
            
        # Sampling settings from env
        if os.getenv("AUTOMAGIK_SAMPLING_RATE"):
            config.default_sampling_rate = float(os.getenv("AUTOMAGIK_SAMPLING_RATE"))
        if os.getenv("AUTOMAGIK_ERROR_SAMPLING_RATE"):
            config.error_sampling_rate = float(os.getenv("AUTOMAGIK_ERROR_SAMPLING_RATE"))
        
        return config

    def is_tracing_enabled(self) -> bool:
        """Check if any tracing is enabled."""
        return not self.disable_all_tracing and (
            self.observability_enabled or self.telemetry_enabled
        )


import os
import logging
from automagik.config import settings, LogLevel

class PrettyFormatter(logging.Formatter):
    """A formatter that adds colors and emojis to log messages."""

    def __init__(self, include_timestamp: bool = True):
        self.include_timestamp = include_timestamp
        format_str = '%(message)s'
        if include_timestamp:
            format_str = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        super().__init__(format_str)
        
        self.colors = {
            logging.INFO: '\033[92m',  # Green
            logging.ERROR: '\033[91m',  # Red
            logging.WARNING: '\033[93m',  # Yellow
            logging.DEBUG: '\033[94m',  # Blue
        }
        self.reset = '\033[0m'

        self.emojis = {
            logging.INFO: '📝',
            logging.ERROR: '❌',
            logging.WARNING: '⚠️',
            logging.DEBUG: '🔍',
        }

    def format(self, record):
        if not record.exc_info:
            level = record.levelno
            if level in self.colors:
                record.msg = f"{self.emojis.get(level, '')} {self.colors[level]}{record.msg}{self.reset}"
        return super().format(record)

def get_log_level(level: LogLevel) -> int:
    """Convert LogLevel enum to logging level."""
    log_levels = {
        LogLevel.DEBUG: logging.DEBUG,
        LogLevel.INFO: logging.INFO,
        LogLevel.WARNING: logging.WARNING,
        LogLevel.ERROR: logging.ERROR,
        LogLevel.CRITICAL: logging.CRITICAL
    }
    return log_levels[level]

def configure_logging():
    """Configure logging with pretty formatting and proper log level."""
    # Get log level from settings
    log_level = get_log_level(settings.AUTOMAGIK_LOG_LEVEL)
    verbose_logging = settings.AUTOMAGIK_VERBOSE_LOGGING
    log_to_file = getattr(settings, 'AUTOMAGIK_LOG_TO_FILE', False)
    log_file_path = getattr(settings, 'AUTOMAGIK_LOG_FILE_PATH', 'debug.log')
    
    # Configure root logger
    root_logger = logging.getLogger()
    root_logger.setLevel(log_level)

    # Remove existing handlers
    for handler in root_logger.handlers[:]:
        root_logger.removeHandler(handler)

    # Create and configure stream handler
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(PrettyFormatter(include_timestamp=verbose_logging))
    root_logger.addHandler(stream_handler)
    
    # Add file handler if enabled
    if log_to_file:
        try:
            file_handler = logging.FileHandler(log_file_path, mode='a')
            # File logging always includes timestamp and no colors
            file_formatter = logging.Formatter(
                '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            )
            file_handler.setFormatter(file_formatter)
            root_logger.addHandler(file_handler)
            print(f"📝 File logging enabled: {log_file_path}")
        except Exception as e:
            print(f"⚠️ Failed to enable file logging: {e}")

    # Configure module-specific log levels
    configure_module_log_levels(verbose_logging)

    # Configure Logfire if token is present
    if settings.AUTOMAGIK_LOGFIRE_TOKEN:
        try:
            import logfire
            os.environ["LOGFIRE_TOKEN"] = settings.AUTOMAGIK_LOGFIRE_TOKEN
            logfire.configure(scrubbing=False)  # Logfire reads token from environment
            logfire.instrument_pydantic_ai()
        except Exception as e:
            print(f"Warning: Failed to configure Logfire: {str(e)}")
    elif not settings.AUTOMAGIK_LOGFIRE_IGNORE_NO_CONFIG:
        print("Warning: AUTOMAGIK_LOGFIRE_TOKEN is not set. Tracing will be disabled.")

def configure_module_log_levels(verbose_logging: bool):
    """Configure log levels for specific modules based on verbosity setting."""
    # Always restrict certain modules regardless of verbosity
    logging.getLogger('httpx').setLevel(logging.WARNING)
    logging.getLogger('httpcore').setLevel(logging.WARNING)
    
    # If not in verbose mode, restrict more modules
    if not verbose_logging:
        # Database operations
        logging.getLogger('automagik.db').setLevel(logging.INFO)
        
        # HTTP requests - restrict details in non-verbose mode
        logging.getLogger('urllib3').setLevel(logging.WARNING)
        
        # API requests in non-verbose mode
        logging.getLogger('automagik.api').setLevel(logging.INFO)
        
        # Memory system - very noisy, restrict to WARNING
        logging.getLogger('automagik.memory').setLevel(logging.WARNING)
        logging.getLogger('automagik.memory.system_prompt_agent_memory_tool').setLevel(logging.WARNING)
        logging.getLogger('automagik.memory.agent_memory_tool').setLevel(logging.WARNING)
        logging.getLogger('automagik.memory.memory_provider').setLevel(logging.WARNING)
        
        # Tool registration noise
        logging.getLogger('automagik.agents.common.tool_registry').setLevel(logging.INFO)
        logging.getLogger('automagik.tools.registry').setLevel(logging.INFO)
        
        # Framework internals
        logging.getLogger('automagik.agents.models.automagik_agent').setLevel(logging.INFO)
        logging.getLogger('automagik.agents.models.dependencies').setLevel(logging.INFO)

def get_logger(name: str) -> logging.Logger:
    """Get a logger with the specified name.
    
    Args:
        name: The name for the logger
        
    Returns:
        Configured logger instance
    """
    return logging.getLogger(name)



from fastapi import HTTPException, Request, Header
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.responses import JSONResponse
from typing import Optional
from automagik.config import settings

API_KEY_NAME = "x-api-key"

class APIKeyMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        # Skip auth for CORS preflight requests (OPTIONS)
        if request.method == "OPTIONS":
            return await call_next(request)
            
        # Skip auth for health check, root, and documentation endpoints
        no_auth_paths = [
            "/health",
            "/",
            "/api/v1/docs",
            "/api/v1/redoc",
            "/api/v1/openapi.json",
            "/api/v1/mcp/health"
        ]
        
        # Check if this path should bypass authentication
        if request.url.path in no_auth_paths:
            return await call_next(request)

        # Check for API key in multiple formats
        api_key = request.headers.get(API_KEY_NAME) or request.query_params.get(API_KEY_NAME)
        
        # Also check for Bearer token (from Swagger UI authorize button)
        if api_key is None:
            auth_header = request.headers.get("authorization")
            if auth_header and auth_header.startswith("Bearer "):
                api_key = auth_header[7:]  # Remove "Bearer " prefix
        
        if api_key is None:
            return JSONResponse(status_code=401, content={"detail": "API key is missing. Provide via x-api-key header, query parameter, or Authorization: Bearer header"})
        if api_key != settings.AUTOMAGIK_API_KEY:
            return JSONResponse(status_code=401, content={"detail": "Invalid API Key"})
            
        return await call_next(request)

async def get_api_key(x_api_key: Optional[str] = Header(None, alias=API_KEY_NAME)):
    """Legacy dependency function for backward compatibility.
    
    Note: This function exposes x-api-key parameter in Swagger docs.
    Use verify_api_key() for routes that should only show Bearer auth.
    """
    if x_api_key is None:
        raise HTTPException(
            status_code=401,
            detail="API key is missing"
        )
    
    if x_api_key != settings.AUTOMAGIK_API_KEY:
        raise HTTPException(
            status_code=401,
            detail="Invalid API key"
        )
    
    return x_api_key

async def verify_api_key():
    """Dependency to validate API key without exposing parameters in Swagger docs.
    
    This function relies on the APIKeyMiddleware to handle authentication,
    so it only validates that authentication has already passed.
    
    Returns:
        True if authentication passed (middleware ensures this)
    """
    # Since the middleware already validated the API key,
    # if we reach this point, authentication has passed
    return True 


"""Privacy-first telemetry collection for anonymous usage analytics."""

from .collector import TelemetryCollector
from .events import EventType, TelemetryEvent

__all__ = [
    'TelemetryCollector',
    'EventType',
    'TelemetryEvent'
]


"""Database provider implementations for PostgreSQL and SQLite support."""

from .base import DatabaseProvider
from .postgresql import PostgreSQLProvider
from .sqlite import SQLiteProvider
from .factory import get_database_provider, create_database_provider

__all__ = [
    "DatabaseProvider",
    "PostgreSQLProvider", 
    "SQLiteProvider",
    "get_database_provider",
    "create_database_provider"
]


"""Memory tool interface helpers.

This module provides helper functions and decorators for memory tools.
"""
from typing import Any, Callable
import logging
import re
from functools import wraps

logger = logging.getLogger(__name__)

def invalidate_memory_cache(func: Callable) -> Callable:
    """Decorator that invalidates the memory cache after the function is called.
    
    This ensures that any memory updates are immediately reflected in
    subsequent system prompt generation.
    
    Args:
        func: The function to decorate
        
    Returns:
        Decorated function
    """
    @wraps(func)
    async def wrapper(*args, **kwargs):
        # Call the original function
        result = await func(*args, **kwargs)
        
        # Get the memory provider
        from automagik.tools.memory.provider import get_memory_provider_for_agent
        
        # Try to extract agent_id from args/kwargs
        agent_id = None
        
        # Check if first argument might be a context with dependencies
        if args and hasattr(args[0], 'deps'):
            deps = args[0].deps
            if hasattr(deps, '_agent_id_numeric'):
                agent_id = deps._agent_id_numeric
                logger.debug(f"Extracted agent_id={agent_id} from args[0].deps._agent_id_numeric")
        
        # Check for context object with dict-like access
        if args and agent_id is None and hasattr(args[0], 'get') and callable(getattr(args[0], 'get')):
            try:
                if args[0].get('agent_id'):
                    agent_id = args[0].get('agent_id')
                    logger.debug(f"Extracted agent_id={agent_id} from args[0].get('agent_id')")
            except Exception as e:
                logger.debug(f"Error accessing context dict: {str(e)}")
        
        # Check if first argument is a dict
        if args and agent_id is None and isinstance(args[0], dict) and 'agent_id' in args[0]:
            agent_id = args[0]['agent_id']
            logger.debug(f"Extracted agent_id={agent_id} from args[0]['agent_id'] direct access")
        
        # Check if context.deps is a dict-like object
        if args and agent_id is None and hasattr(args[0], 'deps') and hasattr(args[0].deps, 'get') and callable(getattr(args[0].deps, 'get')):
            try:
                if args[0].deps.get('agent_id'):
                    agent_id = args[0].deps.get('agent_id')
                    logger.debug(f"Extracted agent_id={agent_id} from args[0].deps.get('agent_id')")
            except Exception as e:
                logger.debug(f"Error accessing deps dict: {str(e)}")
        
        # Check if agent_id is in kwargs
        if agent_id is None and 'agent_id' in kwargs:
            agent_id = kwargs['agent_id']
            logger.debug(f"Extracted agent_id={agent_id} from kwargs['agent_id']")
        
        # Try to extract from the result of the function
        if agent_id is None and isinstance(result, dict) and 'agent_id' in result:
            agent_id = result['agent_id']
            logger.debug(f"Extracted agent_id={agent_id} from result['agent_id']")
        
        # Try to extract the user_id from the context and look up the agent_id
        if agent_id is None and args:
            user_id = None
            
            # Check if context has user_id
            if hasattr(args[0], 'deps') and hasattr(args[0].deps, '_user_id'):
                user_id = args[0].deps._user_id
                logger.debug(f"Found user_id={user_id} from args[0].deps._user_id")
            
            # Try context as dict-like
            if user_id is None and hasattr(args[0], 'get') and callable(getattr(args[0], 'get')):
                try:
                    if args[0].get('user_id'):
                        user_id = args[0].get('user_id')
                        logger.debug(f"Found user_id={user_id} from args[0].get('user_id')")
                except Exception:
                    pass
            
            # Try kwargs
            if user_id is None and 'user_id' in kwargs:
                user_id = kwargs['user_id']
                logger.debug(f"Found user_id={user_id} from kwargs")
            
            # If we found user_id, try to find the current agent for this user
            if user_id:
                try:
                    # Import here to avoid circular imports
                    from automagik.context import get_current_agent_id
                    current_agent_id = get_current_agent_id(user_id)
                    if current_agent_id:
                        agent_id = current_agent_id
                        logger.debug(f"Found agent_id={agent_id} from current agent for user_id={user_id}")
                except Exception as e:
                    logger.debug(f"Error getting current agent ID: {str(e)}")
        
        # If we found an agent_id, invalidate its cache
        if agent_id:
            provider = get_memory_provider_for_agent(agent_id)
            if provider:
                provider.invalidate_cache()
                logger.info(f"Invalidated memory cache for agent {agent_id}")
            else:
                logger.warning(f"No memory provider found for agent {agent_id}")
        else:
            logger.warning(f"Could not determine agent_id for cache invalidation in {func.__name__}")
        
        return result
    
    return wrapper


def validate_memory_name(name: str) -> bool:
    """Validate that a memory name contains only allowed characters.
    
    Args:
        name: Memory name to validate
        
    Returns:
        True if valid, False otherwise
    """
    return bool(re.match(r'^[a-zA-Z0-9_]+$', name))

def format_memory_content(content: Any) -> str:
    """Format memory content for storage.
    
    Args:
        content: Memory content to format
        
    Returns:
        Formatted string representation
    """
    if isinstance(content, str):
        return content
    
    # For other types, convert to string representation
    try:
        import json
        return json.dumps(content)
    except Exception:
        return str(content) 


"""Model Discovery Service for fetching available models from providers."""

import httpx
import logging
from typing import List
from automagik.config import settings

logger = logging.getLogger(__name__)


class ModelDiscoveryService:
    """Fetch available models from providers based on API keys."""
    
    def __init__(self):
        self.settings = settings
    
    async def get_available_models(self) -> List[str]:
        """Returns flat list of model strings like 'openai:gpt-4'."""
        models = []
        
        async with httpx.AsyncClient(timeout=30.0) as client:
            # OpenAI models
            if self.settings.OPENAI_API_KEY:
                try:
                    response = await client.get(
                        "https://api.openai.com/v1/models",
                        headers={"Authorization": f"Bearer {self.settings.OPENAI_API_KEY}"}
                    )
                    if response.status_code == 200:
                        data = response.json()
                        for model in data.get('data', []):
                            model_id = model['id']
                            # Filter out non-chat models
                            if any(x in model_id for x in ['gpt', 'o1', 'o3', 'o4', 'chatgpt']):
                                models.append(f"openai:{model_id}")
                except Exception as e:
                    logger.error(f"Failed to fetch OpenAI models: {e}")
            
            # Anthropic models
            if self.settings.ANTHROPIC_API_KEY:
                try:
                    response = await client.get(
                        "https://api.anthropic.com/v1/models",
                        headers={
                            "x-api-key": self.settings.ANTHROPIC_API_KEY,
                            "anthropic-version": "2023-06-01"
                        }
                    )
                    if response.status_code == 200:
                        data = response.json()
                        models.extend([f"anthropic:{m['id']}" for m in data.get('data', [])])
                except Exception as e:
                    logger.error(f"Failed to fetch Anthropic models: {e}")
            
            # Google Gemini models
            if self.settings.GEMINI_API_KEY:
                try:
                    response = await client.get(
                        f"https://generativelanguage.googleapis.com/v1beta/models?key={self.settings.GEMINI_API_KEY}"
                    )
                    if response.status_code == 200:
                        data = response.json()
                        for m in data.get('models', []):
                            # Only include generation models, not embeddings
                            if 'generateContent' in m.get('supportedGenerationMethods', []):
                                model_name = m['name'].replace('models/', '')
                                models.append(f"google:{model_name}")
                except Exception as e:
                    logger.error(f"Failed to fetch Gemini models: {e}")
        
        return sorted(list(set(models)))  # Return sorted unique models